@@ -18,15 +18,18 @@
 # MA 02110-1301 USA.
 #
 
+from virtconv import _gettext as _
 import virtconv.formats as formats
 import virtconv.vmcfg as vmcfg
 import virtconv.diskcfg as diskcfg
+import virtconv.netdevcfg as netdevcfg
 import virtinst.FullVirtGuest as fv
-
+import virtinst.ImageParser as ImageParser
 from xml.sax.saxutils import escape
-from string import ascii_letters
 import re
 
+ide_letters = list("abcdefghijklmnopqrstuvwxyz")
+
 pv_boot_template = """
   <boot type="xen">
    <guest>
@@ -58,9 +61,7 @@ image_template = """
 <image>
  <name>%(name)s</name>
  <label>%(name)s</label>
- <description>
-  %(description)s
- </description>
+ <description>%(description)s</description>
  <domain>
   %(boot_template)s
   <devices>
@@ -83,7 +84,9 @@ def export_os_params(vm):
     ostype = None
     osvariant = None
 
-    ostype = fv.OS_TYPES.get(vm.os_type)
+    # TODO: Shouldn't be directly using _OS_TYPES here. virt-image libs (
+    # ImageParser?) should handle this info
+    ostype = fv._OS_TYPES.get(vm.os_type)
     if ostype:
         osvariant = ostype.variants.get(vm.os_variant)
 
@@ -152,7 +155,7 @@ def export_disks(vm):
             continue
 
         path = disk.path
-        drive_nr = ascii_letters[int(instance) % 26]
+        drive_nr = ide_letters[int(instance) % 26]
 
         disk_prefix = "xvd"
         if vm.type == vmcfg.VM_TYPE_HVM:
@@ -166,12 +169,12 @@ def export_disks(vm):
         diskout.append("""<drive disk="%s" target="%s%s" />\n""" %
             (path, disk_prefix, drive_nr))
 
-        type = "raw"
+        typ = "raw"
         if disk.type == diskcfg.DISK_TYPE_ISO:
-            type = "iso"
+            typ = "iso"
         storage.append(
             """<disk file="%s" use="system" format="%s"/>\n""" %
-                (path, type))
+                (path, typ))
 
     return storage, diskout
 
@@ -181,16 +184,20 @@ class virtimage_parser(formats.parser):
     """
     name = "virt-image"
     suffix = ".virt-image.xml"
-    can_import = False
+    can_import = True
     can_export = True
-    can_identify = False
+    can_identify = True
 
     @staticmethod
     def identify_file(input_file):
         """
         Return True if the given file is of this format.
         """
-        raise NotImplementedError
+        try:
+            ImageParser.parse_file(input_file)
+        except ImageParser.ParserException:
+            return False
+        return True
 
     @staticmethod
     def import_file(input_file):
@@ -198,24 +205,68 @@ class virtimage_parser(formats.parser):
         Import a configuration file.  Raises if the file couldn't be
         opened, or parsing otherwise failed.
         """
-        raise NotImplementedError
+        vm = vmcfg.vm()
+        try:
+            config  = ImageParser.parse_file(input_file)
+        except Exception, e:
+            raise ValueError(_("Couldn't import file '%s': %s") %
+                             (input_file, e))
+
+        domain = config.domain
+        boot = domain.boots[0]
+
+        if not config.name:
+            raise ValueError(_("No Name defined in '%s'") % input_file)
+        vm.name = config.name
+        vm.memory = int(config.domain.memory / 1024)
+        if config.descr:
+            vm.description = config.descr
+        vm.nr_vcpus = config.domain.vcpu
+
+        bus = "ide"
+        nr_disk = 0
+
+        for d in boot.drives:
+            disk = d.disk
+            format = None
+            if disk.format == ImageParser.Disk.FORMAT_RAW:
+                format = diskcfg.DISK_FORMAT_RAW
+            elif format == ImageParser.Disk.FORMAT_VMDK:
+                format = diskcfg.DISK_FORMAT_VMDK
+
+            if format is None:
+                raise ValueError(_("Unable to determine disk format"))
+            devid = (bus, nr_disk)
+            vm.disks[devid] = diskcfg.disk(bus = bus,
+                type = diskcfg.DISK_TYPE_DISK)
+            vm.disks[devid].format = format
+            vm.disks[devid].path = disk.file
+            nr_disk = nr_disk + 1
+
+        nics = domain.interface
+        nic_idx = 0
+        while nic_idx in range(0, nics):
+            vm.netdevs[nic_idx] = netdevcfg.netdev(type = netdevcfg.NETDEV_TYPE_UNKNOWN)
+            nic_idx = nic_idx + 1
+            """  Eventually need to add support for mac addresses if given"""
+        vm.validate()
+        return vm
 
     @staticmethod
-    def export_file(vm, output_file):
+    def export(vm):
         """
-        Export a configuration file.
+        Export a configuration file as a string.
         @vm vm configuration instance
-        @file Output file
 
-        Raises ValueError if configuration is not suitable, or another
-        exception on failure to write the output file.
+        Raises ValueError if configuration is not suitable.
         """
 
         if not vm.memory:
-            raise ValueError("VM must have a memory setting")
+            raise ValueError(_("VM must have a memory setting"))
 
-        # xend wants the name to match r'^[A-Za-z0-9_\-\.\:\/\+]+$'
-        vmname = re.sub(r'[^A-Za-z0-9_.:/+-]+',  '_', vm.name)
+        # xend wants the name to match r'^[A-Za-z0-9_\-\.\:\/\+]+$', and
+        # the schema agrees.
+        vmname = re.sub(r'[^A-Za-z0-9_\-\.:\/\+]+',  '_', vm.name)
 
         # Hmm.  Any interface is a good interface?
         interface = None
@@ -249,8 +300,22 @@ class virtimage_parser(formats.parser):
             "storage" : "".join(storage),
         }
 
+        return out
+
+    @staticmethod
+    def export_file(vm, output_file):
+        """
+        Export a configuration file.
+        @vm vm configuration instance
+        @output_file Output file
+
+        Raises ValueError if configuration is not suitable, or another
+        exception on failure to write the output file.
+        """
+        output = virtimage_parser.export(vm)
+
         outfile = open(output_file, "w")
-        outfile.writelines(out)
+        outfile.writelines(output)
         outfile.close()
 
 formats.register_parser(virtimage_parser)
