@@ -17,16 +17,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
+"""
+Module for cloning an existing virtual machine
+
+General workflow for cloning:
+
+    - Instantiate CloneDesign. Requires at least a libvirt connection and
+      a name of a domain to clone.
+
+    - Run 'setup' from the CloneDesign instance to prep for cloning
+
+    - Run 'CloneManager.start_duplicate', passing the CloneDesign instance
+"""
 
 import os
-import sys
-import stat
-import re
 import libxml2
 import logging
-import urlgrabber.progress as progress
-import util
-import commands
+import subprocess
+import _util
 import libvirt
 import Guest
 from VirtualDisk import VirtualDisk
@@ -37,11 +45,11 @@ from virtinst import _virtinst as _
 #
 class CloneDesign(object):
 
-    def __init__(self,connection):
+    def __init__(self, connection):
         # hypervisor connection
         self._hyper_conn = connection
 
-        # original guest name or uuid 
+        # original guest name or uuid
         self._original_guest        = None
         self._original_dom          = None
         self._original_devices      = []
@@ -49,7 +57,10 @@ class CloneDesign(object):
         self._original_devices_type = []
         self._original_xml          = None
 
-        # clone guest 
+        # Deliberately private: user doesn't need to know this
+        self._original_devices_idx = []
+
+        # clone guest
         self._clone_name         = None
         self._clone_devices      = []
         self._clone_devices_size = []
@@ -67,18 +78,21 @@ class CloneDesign(object):
         # Throwaway guest to use for easy validation
         self._valid_guest        = Guest.Guest(connection=connection)
 
+
+    # Getter/Setter methods
+
     def get_original_guest(self):
         return self._original_guest
     def set_original_guest(self, original_guest):
         if type(original_guest) is not type("str") or len(original_guest)==0:
-           raise ValueError, _("Name or UUID of guest to clone is required")
+            raise ValueError, _("Name or UUID of guest to clone is required")
 
         try:
             self._valid_guest.set_uuid(original_guest)
-        except ValueError, e:
+        except ValueError:
             try:
                 self._valid_guest.set_name(original_guest)
-            except ValueError, e:
+            except ValueError:
                 raise ValueError, \
                     _("A valid name or UUID of guest to clone is required")
         self._original_guest = original_guest
@@ -105,14 +119,16 @@ class CloneDesign(object):
     clone_uuid = property(get_clone_uuid, set_clone_uuid)
 
     def set_clone_devices(self, devices):
+        # Devices here is a string path. Every call to set_clone_devices
+        # Adds the path (if valid) to the internal _clone_devices list
         if len(devices) == 0:
             raise ValueError, _("New file to use for disk image is required")
-        cdev      = []
-        cdev_size = []
-        cdev_type = []
-        cdev.append(devices)
-        cdev_size,\
-        cdev_type = self._get_clone_devices_info(cdev)
+
+        # Check path's size (if present)
+        # XXX: Only works locally
+        cdev_size, dummy = self._local_paths_info([devices])
+
+        # Make sure path is valid and we can use it
         devices = self._check_file(self._hyper_conn, devices, cdev_size[0])
         self._clone_devices.append(devices)
     def get_clone_devices(self):
@@ -120,7 +136,7 @@ class CloneDesign(object):
     clone_devices = property(get_clone_devices, set_clone_devices)
 
     def set_clone_mac(self, mac):
-        Guest.VirtualNetworkInterface(mac)
+        Guest.VirtualNetworkInterface(mac, conn=self.original_conn)
         self._clone_mac.append(mac)
     def get_clone_mac(self):
         return self._clone_mac
@@ -176,92 +192,104 @@ class CloneDesign(object):
         return self._force_target
     force_target = property(set_force_target)
 
-    #
-    # setup original guest
-    #
+
+    # Functional methods
+
     def setup_original(self):
-        logging.debug("setup_original in")
+        """
+        Validate and setup all parameters needed for the original (cloned) VM
+        """
+        logging.debug("Validating original guest parameters")
+
         try:
             self._original_dom = self._hyper_conn.lookupByName(self._original_guest)
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError:
             raise RuntimeError, _("Domain %s is not found") % self._original_guest
 
-        #
-        # store the xml as same as original xml still setup_clone_xml
-        #
+        # For now, clone_xml is just a copy of the original
         self._original_xml = self._original_dom.XMLDesc(0)
         self._clone_xml    = self._original_dom.XMLDesc(0)
+
+        # Pull clonable storage info from the original xml
         self._original_devices,     \
         self._original_devices_size,\
-        self._original_devices_type = self._get_original_devices_info(self._original_xml)
+        self._original_devices_type,\
+        self._original_devices_idx = self._get_original_devices_info(self._original_xml)
 
-        #
-        # check status. Firt, shut off domain is available.
-        #
+        logging.debug("Original paths: %s" % (self._original_devices))
+        logging.debug("Original sizes: %s" % (self._original_devices_size))
+        logging.debug("Original types: %s" % (self._original_devices_type))
+        logging.debug("Original idxs: %s" % (self._original_devices_idx))
+
+        # Check original domain is SHUTOFF
+        # XXX: Shouldn't pause also be fine, and guests with no storage/
+        # XXX: readonly + sharable storage can be cloned while running
         status = self._original_dom.info()[0]
         logging.debug("original guest status: %s" % (status))
         if status != libvirt.VIR_DOMAIN_SHUTOFF:
             raise RuntimeError, _("Domain status must be SHUTOFF")
 
-        #
-        # check existing
-        #
+        # Make sure new VM name isn't taken.
+        # XXX: Check this at set time?
         try:
             if self._hyper_conn.lookupByName(self._clone_name) is not None:
                 raise RuntimeError, _("Domain %s already exists") % self._clone_name
         except libvirt.libvirtError:
             pass
 
-        #
-        # check used uuid
-        # random uuid check is done in start_duplicate function
-        #
-        if self._check_uuid(self._clone_uuid) == True:
-            raise RuntimeError, _("The UUID you entered is already in use by another guest!")
+        # Check specified UUID isn't taken
+        if _util.vm_uuid_collision(self._hyper_conn, self._clone_uuid):
+            raise RuntimeError, _("The UUID you entered is already in use by "
+                                  "another guest!")
 
-        #
-        # check used mac
-        #
+        # Check mac address is not in use
+        # XXX: Check this at set time?
         for i in self._clone_mac:
             ret, msg = self._check_mac(i)
             if msg is not None:
                 if ret:
                     raise RuntimeError, msg
                 else:
-                    print >> sys.stderr, msg
                     logging.warning(msg)
 
-        logging.debug("setup_original out")
 
-    #
-    # setup clone XML
-    #
     def setup_clone(self):
-        logging.debug("setup_clone in")
+        """
+        Validate and set up all parameters needed for the new (clone) VM
+        """
+        logging.debug("Validating clone parameters.")
 
+        # XXX: Make sure a clone name has been specified? or generate one?
+
+        # XXX: Only works locally
         self._clone_devices_size,\
-        self._clone_devices_type = self._get_clone_devices_info(self._clone_devices)
+        self._clone_devices_type = self._local_paths_info(self._clone_devices)
 
+        logging.debug("Clone paths: %s" % (self._clone_devices))
+        logging.debug("Clone sizes: %s" % (self._clone_devices_size))
+        logging.debug("Clone types: %s" % (self._clone_devices_type))
+
+        # We simply edit the original VM xml in place
+        # XXX: Does this need a huge try except so we don't leak xml memory
+        # XXX: on failure?
         doc = libxml2.parseDoc(self._clone_xml)
         ctx = doc.xpathNewContext()
-        type = ctx.xpathEval("/domain")[0].prop("type")
+        typ = ctx.xpathEval("/domain")[0].prop("type")
 
         # changing name
         node = ctx.xpathEval("/domain/name")
         node[0].setContent(self._clone_name)
 
-        # changing devices
+        # Changing storage paths
         clone_devices = iter(self._clone_devices)
-        count = ctx.xpathEval("count(/domain/devices/disk)")
-        for i in range(1, int(count+1)):
-            node = self._get_available_cloning_device(ctx, i, self._force_target)
-            if node == None:
-                continue
+        for i in self._original_devices_idx:
+            node = ctx.xpathEval("/domain/devices/disk[%d]/source" % i)
             node = node[0].get_properties()
             try:
                 node.setContent(clone_devices.next())
-            except Exception, e:
-                raise ValueError, _("Missing new file to use disk image for %s") % node.getContent()
+            except Exception:
+                raise ValueError, _("Missing new file to use disk image "
+                                    "for %s") % node.getContent()
 
         # changing uuid
         node = ctx.xpathEval("/domain/uuid")
@@ -269,11 +297,10 @@ class CloneDesign(object):
             node[0].setContent(self._clone_uuid)
         else:
             while 1:
-                uuid = util.uuidToString(util.randomUUID())
-                if self._check_uuid(uuid) == True:
+                uuid = _util.uuidToString(_util.randomUUID())
+                if _util.vm_uuid_collision(self._hyper_conn, uuid):
                     continue
-                else:
-                    break
+                break
             node[0].setContent(uuid)
 
         # changing mac
@@ -282,150 +309,101 @@ class CloneDesign(object):
             node = ctx.xpathEval("/domain/devices/interface[%d]/mac/@address" % i)
             try:
                 node[0].setContent(self._clone_mac[i-1])
-            except Exception, e:
+            except Exception:
                 while 1:
-                    mac = util.randomMAC(type)
-                    ret, msg = self._check_mac(mac)
+                    mac = _util.randomMAC(typ)
+                    dummy, msg = self._check_mac(mac)
                     if msg is not None:
                         continue
                     else:
                         break
                 node[0].setContent(mac)
 
-        # change disk type
-        self._change_disk_type(self._original_devices_type, self._clone_devices_type, ctx)
+        # Change xml disk type values if original and clone disk types
+        # (block/file) don't match
+        self._change_disk_type(self._original_devices_type,
+                               self._clone_devices_type,
+                               self._original_devices_idx,
+                               ctx)
 
-        # set clone xml
+        # Save altered clone xml
         self._clone_xml = str(doc)
 
         ctx.xpathFreeContext()
         doc.freeDoc()
-        logging.debug("setup_clone out")
 
-    #
-    # setup
-    #
+
     def setup(self):
+        """
+        Helper function that wraps setup_original and setup_clone, with
+        additional debug logging.
+        """
         self.setup_original()
-        logging.debug("original guest is\n%s" % (self._original_xml))
+        logging.debug("Original guest xml is\n%s" % (self._original_xml))
 
         self.setup_clone()
-        logging.debug("cloning guest is\n%s" % (self._clone_xml))
-  
-    #
-    # check used uuid func
-    # False : OK
-    # True  : NG existing
-    #
-    def _check_uuid(self, uuid):
-        check = False
-        if uuid is not None:
-            try:
-                if self._hyper_conn.lookupByUUIDString(uuid) is not None:
-                    check = True
-                else:
-                    pass
-            except libvirt.libvirtError, e:
-                pass
-        return check
+        logging.debug("Clone guest xml is\n%s" % (self._clone_xml))
 
-    #
-    # check used file func
-    # ret : Use File Path
-    #
+
+    # Private helper functions
+
+    # Check if new file path is valid
     def _check_file(self, conn, disk, size):
-        d = VirtualDisk(disk, size)
+        d = VirtualDisk(disk, size, conn=conn)
         return d.path
 
-    #
-    # check used mac func
-    #
+    # Check if new mac address is valid
     def _check_mac(self, mac):
-        nic = Guest.VirtualNetworkInterface(macaddr=mac)
+        nic = Guest.VirtualNetworkInterface(macaddr=mac,
+                                            conn=self.original_conn)
         return nic.is_conflict_net(self._hyper_conn)
 
-    #
-    # get count macaddr
-    #
-    def _count_mac(self, vms, mac):
-        count = 0
-        for vm in vms:
-            doc = None
-            try:
-                doc = libxml2.parseDoc(vm.XMLDesc(0))
-            except:
-                continue
-            ctx = doc.xpathNewContext()
-            mac_index = (str(doc).upper()).find(mac.upper())
-            if mac_index == -1:
-                continue
-            mac_comp = str(doc)[mac_index:mac_index+17]
-            try:
-                try:
-                    count += ctx.xpathEval("count(/domain/devices/interface/mac[@address='%s'])"
-                                           % mac_comp)
-                except:
-                    continue
-            finally:
-                if ctx is not None:
-                    ctx.xpathFreeContext()
-                if doc is not None:
-                    doc.freeDoc()
-        return count
-
-    #
-    # get the original devices information 
-    #
+    # Parse disk paths that need to be cloned from the original guest's xml
+    # Return a tuple of lists:
+    # ([list of paths to clone], [size of those paths],
+    #  [file/block type of those paths], [indices of disks to be cloned])
     def _get_original_devices_info(self, xml):
 
-        list = []
+        lst  = []
         size = []
-        type = []
+        typ  = []
+        idx_lst = []
 
         doc = libxml2.parseDoc(xml)
         ctx = doc.xpathNewContext()
         try:
             count = ctx.xpathEval("count(/domain/devices/disk)")
             for i in range(1, int(count+1)):
+                # Check if the disk needs cloning
                 node = self._get_available_cloning_device(ctx, i, self._force_target)
                 if node == None:
                     continue
-                list.append(node[0].get_properties().getContent())
+                idx_lst.append(i)
+                lst.append(node[0].get_properties().getContent())
         finally:
             if ctx is not None:
                 ctx.xpathFreeContext()
             if doc is not None:
                 doc.freeDoc()
-        logging.debug("original device list: %s" % (list))
 
-        for i in list:
-            mode = os.stat(i)[stat.ST_MODE]
-            if stat.S_ISBLK(mode):
-                ret,str = commands.getstatusoutput('fdisk -s %s' % i)
-                # check
-                if str.isdigit() == False:
-                    lines = str.splitlines()
-                    # retry eg. for the GPT disk
-                    str = lines[len(lines)-1]
-                size.append(int(str) * 1024)
-                type.append(False)
-            elif stat.S_ISREG(mode):
-                size.append(os.path.getsize(i))
-                type.append(True)
-        logging.debug("original device size: %s" % (size))
-        logging.debug("original device type: %s" % (type))
+        # Lookup size and storage type (file/block)
+        for i in lst:
+            (t, sz) = _util.stat_disk(i)
+            typ.append(t)
+            size.append(sz)
 
-        return (list, size, type)
+        return (lst, size, typ, idx_lst)
 
-    #
-    # get available original device for clone
-    #
+    # Pull disk #i from the original guest xml, return it's xml
+    # if it should be cloned (skips readonly, empty, or sharable disks
+    # unless its target is in the 'force' list)
     def _get_available_cloning_device(self, ctx, i, force):
 
         node = None
         force_flg = False
-        
+
         node = ctx.xpathEval("/domain/devices/disk[%d]/source" % i)
+        # If there is no media path, ignore
         if len(node) == 0:
             return None
 
@@ -436,98 +414,93 @@ class CloneDesign(object):
             if target == f_target:
                 force_flg = True
 
+        # Skip readonly disks unless forced
         ro = ctx.xpathEval("/domain/devices/disk[%d]/readonly" % i)
         if len(ro) != 0 and force_flg == False:
             return None
+        # Skip sharable disks unless forced
         share = ctx.xpathEval("/domain/devices/disk[%d]/shareable" % i) 
         if len(share) != 0 and force_flg == False:
             return None
 
-        return node 
+        return node
 
-    #
-    # get the clone devices information
-    #
-    def _get_clone_devices_info(self, cln_dev_lst):
+    # Stat each path in the passed list, return a tuple of
+    # ([size of each path (0 if non-existent)],
+    #  [file/block type of each path (true for 'file', false for 'block')]
+    def _local_paths_info(self, paths_lst):
 
         size = []
-        type = []
+        typ  = []
 
-        for i in cln_dev_lst:
-            if os.path.exists(i) ==  False:
-               size.append(0)
-               # if not exists, create file necessary
-               type.append(True)
-               continue
-            mode = os.stat(i)[stat.ST_MODE]
-            if stat.S_ISBLK(mode):
-                ret,str = commands.getstatusoutput('fdisk -s %s' % i)
-                # check
-                if str.isdigit() == False:
-                    lines = str.splitlines()
-                    # retry eg. for the GPT disk
-                    str = lines[len(lines)-1]
-                size.append(int(str) * 1024)
-                type.append(False)
-            elif stat.S_ISREG(mode):
-                size.append(os.path.getsize(i))
-                type.append(True)
+        for i in paths_lst:
+            (t, sz) = _util.stat_disk(i)
+            typ.append(t)
+            size.append(sz)
 
-        logging.debug("clone device list: %s" % (cln_dev_lst))
-        logging.debug("clone device size: %s" % (size))
-        logging.debug("clone device type: %s" % (type))
+        return (size, typ)
 
-        return (size, type)
+    # Check if original disk type (file/block) is different from
+    # requested clones disk type, and alter xml if needed
+    def _change_disk_type(self, org_type, cln_type, idxs, ctx):
 
-    #
-    # change disk type in XML
-    #
-    def _change_disk_type(self, org_type, cln_type, ctx):
-        for i in range(len(org_type)):
-            disk_type = ctx.xpathEval("/domain/devices/disk[%d]/@type" % (i+1))
-            driv_name = ctx.xpathEval("/domain/devices/disk[%d]/driver/@name" % (i+1))
-
-            src = ctx.xpathEval("/domain/devices/disk[%d]/source" % (i+1))
+        type_idx = 0
+        for dev_idx in idxs:
+            disk_type = ctx.xpathEval("/domain/devices/disk[%d]/@type" %
+                                      dev_idx)
+            driv_name = ctx.xpathEval("/domain/devices/disk[%d]/driver/@name" % dev_idx)
+            src = ctx.xpathEval("/domain/devices/disk[%d]/source" % dev_idx)
             src_chid_txt = src[0].get_properties().getContent()
 
             # different type
-            if org_type[i] != cln_type[i]:
-                # changing from file to disk
-                if org_type[i] == True:
-                    disk_type[0].setContent("block")
-                    driv_name[0].setContent("phy")
-                    src[0].get_properties().unlinkNode()
-                    src[0].newProp("dev", src_chid_txt)
-                # changing from disk to file
+            if org_type[type_idx] != cln_type[type_idx]:
+                if org_type[type_idx] == True:
+                    # changing from file to disk
+                    typ, driv, newprop = ("block", "phy", "dev")
                 else:
-                    disk_type[0].setContent("file")
-                    driv_name[0].setContent("file")
-                    src[0].get_properties().unlinkNode()
-                    src[0].newProp("file", src_chid_txt)
+                    # changing from disk to file
+                    typ, driv, newprop = ("file", "file", "file")
+
+                disk_type[0].setContent(typ)
+                if driv_name:
+                    driv_name[0].setContent(driv)
+                src[0].get_properties().unlinkNode()
+                src[0].newProp(newprop, src_chid_txt)
+
+            type_idx += 1
 
 #
 # start duplicate
 # this function clones the virtual machine according to the ClonDesign object
 #
-def start_duplicate(design):
+def start_duplicate(design, meter=None):
 
     logging.debug("start_duplicate in")
 
     # do dupulicate
     # at this point, handling the cloning way.
     if design.preserve == True:
-        _do_duplicate(design)
+        _do_duplicate(design, meter)
 
     # define clone xml
     design.original_conn.defineXML(design.clone_xml)
 
     logging.debug("start_duplicate out")
 
+def _vdisk_clone(path, clone):
+    path = os.path.expanduser(path)
+    clone = os.path.expanduser(clone)
+    try:
+        rc = subprocess.call([ '/usr/sbin/vdiskadm', 'clone', path, clone ])
+        return rc == 0
+    except OSError:
+        return False
+
 #
 # Now this Cloning method is reading and writing devices.
 # For future, there are many cloning methods (e.g. fork snapshot cmd).
 #
-def _do_duplicate(design):
+def _do_duplicate(design, meter):
 
     src_fd = None
     dst_fd = None
@@ -538,20 +511,28 @@ def _do_duplicate(design):
     sparse_copy_mode = False
 
     try:
-        for src_dev in design.original_devices: 
+        for src_dev in design.original_devices:
             dst_dev = dst_dev_iter.next()
             dst_siz = dst_siz_iter.next()
 
-            size = dst_siz
-            meter = progress.TextMeter()
-            print _("Cloning from %(src)s to %(dst)s") % {'src' : src_dev, \
-                                                       'dst' : dst_dev}
-            meter.start(size=size, text=_("Cloning domain..."))
+            meter.start(size=dst_siz,
+                        text=_("Cloning from %(src)s to %(dst)s...") % \
+                        {'src' : src_dev, 'dst' : dst_dev})
 
-            # skip
             if src_dev == "/dev/null" or src_dev == dst_dev:
-                meter.end(size)
+                meter.end(dst_siz)
                 continue
+
+            # vdisk specific handlings
+            if _util.is_vdisk(src_dev) or (os.path.exists(dst_dev) and
+                                           _util.is_vdisk(dst_dev)):
+                if not _util.is_vdisk(src_dev) or os.path.exists(dst_dev):
+                    raise RuntimeError, _("copying to an existing vdisk is not supported")
+                if not _vdisk_clone(src_dev, dst_dev):
+                    raise RuntimeError, _("failed to clone disk")
+                meter.end(dst_siz)
+                continue
+
             #
             # create sparse file
             # if a destination file exists and sparse flg is True,
@@ -561,8 +542,7 @@ def _do_duplicate(design):
                 design.clone_bs = 4096
                 sparse_copy_mode = True
                 fd = os.open(dst_dev, os.O_WRONLY | os.O_CREAT)
-                os.lseek(fd, dst_siz, 0)
-                os.write(fd, '\x00')
+                os.ftruncate(fd, dst_siz)
                 os.close(fd)
             else:
                 design.clone_bs = 1024*1024*10
@@ -577,7 +557,7 @@ def _do_duplicate(design):
                 l = os.read(src_fd, design.clone_bs)
                 s = len(l)
                 if s == 0:
-                    meter.end(size)
+                    meter.end(dst_siz)
                     break
                 # check sequence of zeros
                 if sparse_copy_mode == True and zeros == l:
@@ -588,7 +568,7 @@ def _do_duplicate(design):
                         meter.end(i)
                         break
                 i += s
-                if i < size:
+                if i < dst_siz:
                     meter.update(i)
 
             os.close(src_fd)
@@ -597,7 +577,7 @@ def _do_duplicate(design):
             dst_fd = None
     finally:
         if src_fd is not None:
-           os.close(src_fd)
+            os.close(src_fd)
         if dst_fd is not None:
-           os.close(dst_fd)
+            os.close(dst_fd)
 
