@@ -20,28 +20,37 @@
 # MA 02110-1301 USA.
 
 import os, os.path
-import statvfs
-import stat, sys, time
+import errno
+import struct
+import time
 import re
 import libxml2
 import urlgrabber.progress as progress
-import util
+import _util
 import libvirt
+import platform
 import __builtin__
 import CapabilitiesParser
+import VirtualDevice
+
+import osdict
 from VirtualDisk import VirtualDisk
 from virtinst import _virtinst as _
 
 import logging
 
-class VirtualNetworkInterface(object):
+XEN_SCRATCH="/var/lib/xen"
+LIBVIRT_SCRATCH="/var/lib/libvirt/boot"
+
+class VirtualNetworkInterface(VirtualDevice.VirtualDevice):
 
     TYPE_BRIDGE  = "bridge"
     TYPE_VIRTUAL = "network"
     TYPE_USER    = "user"
 
     def __init__(self, macaddr=None, type=TYPE_BRIDGE, bridge=None,
-                 network=None, model=None):
+                 network=None, model=None, conn=None):
+        VirtualDevice.VirtualDevice.__init__(self, conn)
 
         if macaddr is not None and \
            __builtin__.type(macaddr) is not __builtin__.type("string"):
@@ -67,6 +76,29 @@ class VirtualNetworkInterface(object):
         else:
             raise ValueError, _("Unknown network type %s") % (type,)
 
+    def get_network(self):
+        return self._network
+    def set_network(self, newnet):
+        def _is_net_active(netobj):
+            """Apparently the 'info' command was never hooked up for
+               libvirt virNetwork python apis."""
+            if not self.conn:
+                return True
+            return self.conn.listNetworks().count(netobj.name())
+
+        if newnet is not None and self.conn:
+            try:
+                net = self.conn.networkLookupByName(newnet)
+            except libvirt.libvirtError, e:
+                raise ValueError(_("Virtual network '%s' does not exist: %s") \
+                                   % (newnet, str(e)))
+            if not _is_net_active(net):
+                raise ValueError(_("Virtual network '%s' has not been "
+                                   "started.") % newnet)
+
+        self._network = newnet
+    network = property(get_network, set_network)
+
     def is_conflict_net(self, conn):
         """is_conflict_net: determines if mac conflicts with others in system
 
@@ -78,15 +110,15 @@ class VirtualNetworkInterface(object):
         if self.macaddr is None:
             return (False, None)
         # get Running Domains
-        ids = conn.listDomainsID();
+        ids = conn.listDomainsID()
         vms = []
-        for id in ids:
+        for i in ids:
             try:
-                vm = conn.lookupByID(id)
+                vm = conn.lookupByID(i)
                 vms.append(vm)
             except libvirt.libvirtError:
                 # guest probably in process of dieing
-                logging.warn("conflict_net: Failed to lookup domain id %d" % id)
+                logging.warn("conflict_net: Failed to lookup domain id %d" % i)
         # get inactive Domains
         inactive_vm = []
         names = conn.listDefinedDomains()
@@ -99,7 +131,7 @@ class VirtualNetworkInterface(object):
                 logging.warn("conflict_net: Failed to lookup domain %d" % name)
 
         # get the Host's NIC MACaddress
-        hostdevs = util.get_host_network_devices()
+        hostdevs = _util.get_host_network_devices()
 
         if self.countMACaddr(vms) > 0:
             return (True, _("The MAC address you entered is already in use by another active virtual machine."))
@@ -113,7 +145,7 @@ class VirtualNetworkInterface(object):
     def setup(self, conn):
         if self.macaddr is None:
             while 1:
-                self.macaddr = util.randomMAC(type=conn.getType().lower())
+                self.macaddr = _util.randomMAC(type=conn.getType().lower())
                 if self.is_conflict_net(conn)[1] is not None:
                     continue
                 else:
@@ -127,7 +159,7 @@ class VirtualNetworkInterface(object):
                     raise RuntimeError(msg)
 
         if not self.bridge and self.type == "bridge":
-            self.bridge = util.default_bridge()
+            self.bridge = _util.default_bridge()
 
     def get_xml_config(self):
         src_xml = ""
@@ -147,6 +179,8 @@ class VirtualNetworkInterface(object):
                "    </interface>"
 
     def countMACaddr(self, vms):
+        if not self.macaddr:
+            return
         count = 0
         for vm in vms:
             doc = None
@@ -158,7 +192,7 @@ class VirtualNetworkInterface(object):
             try:
                 for mac in ctx.xpathEval("/domain/devices/interface/mac"):
                     macaddr = mac.xpathEval("attribute::address")[0].content
-                    if util.compareMAC(self.macaddr, macaddr) == 0:
+                    if macaddr and _util.compareMAC(self.macaddr, macaddr) == 0:
                         count += 1
             finally:
                 if ctx is not None:
@@ -216,7 +250,7 @@ class VirtualGraphics(object):
         return self._keymap
     def set_keymap(self, val):
         if not val:
-            val = util.default_keymap()
+            val = _util.default_keymap()
         if not val or type(val) != type("string"):
             raise ValueError, _("Keymap must be a string")
         if len(val) > 16:
@@ -293,12 +327,6 @@ class Installer(object):
 
         self._tmpfiles = []
 
-    def cleanup(self):
-        for f in self._tmpfiles:
-            logging.debug("Removing " + f)
-            os.unlink(f)
-        self._tmpfiles = []
-
     def get_install_disk(self):
         return self._install_disk
     install_disk = property(get_install_disk)
@@ -320,10 +348,14 @@ class Installer(object):
     os_type = property(get_os_type, set_os_type)
 
     def get_scratchdir(self):
-        if self.type == "xen":
-            if os.path.exists("/var/lib/xen"):
-                return "/var/lib/xen"
-        return "/var/lib/libvirt/boot"
+        if platform.system() == 'SunOS':
+            return '/var/tmp'
+        if self.type == "xen" and os.path.exists(XEN_SCRATCH):
+            return XEN_SCRATCH
+        if os.geteuid() == 0 and os.path.exists(LIBVIRT_SCRATCH):
+            return LIBVIRT_SCRATCH
+        else:
+            return os.path.expanduser("~/.virtinst/boot")
     scratchdir = property(get_scratchdir)
 
     def get_cdrom(self):
@@ -369,8 +401,123 @@ class Installer(object):
         self._extraargs = val
     extraargs = property(get_extra_args, set_extra_args)
 
+    # Private methods
+
+    def _get_osblob_helper(self, isinstall, ishvm, arch=None, loader=None,
+                           conn=None, kernel=None, bootdev=None):
+        osblob = ""
+        if not isinstall and not ishvm:
+            return "<bootloader>%s</bootloader>" % _util.pygrub_path(conn)
+
+        osblob = "<os>\n"
+
+        os_type = self.os_type
+        # Hack for older libvirt Xen driver
+        if os_type == "xen" and self.type == "xen":
+            os_type = "linux"
+
+        if arch:
+            osblob += "    <type arch='%s'>%s</type>\n" % (arch, os_type)
+        else:
+            osblob += "    <type>%s</type>\n" % os_type
+
+        if loader:
+            osblob += "    <loader>%s</loader>\n" % loader
+
+        if isinstall and kernel and kernel["kernel"]:
+            osblob += "    <kernel>%s</kernel>\n"   % _util.xml_escape(kernel["kernel"])
+            osblob += "    <initrd>%s</initrd>\n"   % _util.xml_escape(kernel["initrd"])
+            osblob += "    <cmdline>%s</cmdline>\n" % _util.xml_escape(kernel["extraargs"])
+        elif bootdev is not None:
+            osblob += "    <boot dev='%s'/>\n" % bootdev
+
+        osblob += "  </os>"
+
+        return osblob
+
+
+    # Method definitions
+
+    def cleanup(self):
+        """
+        Remove any temporary files retrieved during installation
+        """
+        for f in self._tmpfiles:
+            logging.debug("Removing " + f)
+            os.unlink(f)
+        self._tmpfiles = []
+
+    def prepare(self, guest, meter, distro=None):
+        """
+        Fetch any files needed for installation.
+        @param guest: guest instance being installed
+        @type L{Guest}
+        @param meter: progress meter
+        @type Urlgrabber ProgressMeter
+        @param distro: Name of distro being installed
+        @type C{str} name from Guest os dictionary
+        """
+        raise NotImplementedError("Must be implemented in subclass")
+
+    def post_install_check(self, guest):
+        """
+        Attempt to verify that installing to disk was successful.
+        @param guest: guest instance that was installed
+        @type L{Guest}
+        """
+
+        if _util.is_uri_remote(guest.conn.getURI()):
+            # XXX: Use block peek for this?
+            return True
+
+        if len(guest.disks) == 0 \
+           or guest.disks[0].device != VirtualDisk.DEVICE_DISK:
+            return True
+
+        if _util.is_vdisk(guest.disks[0].path):
+            return True
+
+        # Check for the 0xaa55 signature at the end of the MBR
+        try:
+            fd = os.open(guest.disks[0].path, os.O_RDONLY)
+        except OSError, (err, msg):
+            logging.debug("Failed to open guest disk: %s" % msg)
+            if err == errno.EACCES and os.geteuid() != 0:
+                return True # non root might not have access to block devices
+            else:
+                raise
+        buf = os.read(fd, 512)
+        os.close(fd)
+        return (len(buf) == 512 and
+                struct.unpack("H", buf[0x1fe: 0x200]) == (0xaa55,))
+
 class Guest(object):
-    def __init__(self, type=None, connection=None, hypervisorURI=None, installer=None):
+
+    # OS Dictionary static variables and methods
+    _DEFAULTS = osdict.DEFAULTS
+    _OS_TYPES = osdict.OS_TYPES
+
+    def list_os_types():
+        return osdict.sort_helper(Guest._OS_TYPES)
+    list_os_types = staticmethod(list_os_types)
+
+    def list_os_variants(type):
+        return osdict.sort_helper(Guest._OS_TYPES[type]["variants"])
+    list_os_variants = staticmethod(list_os_variants)
+
+    def get_os_type_label(type):
+        return Guest._OS_TYPES[type]["label"]
+    get_os_type_label = staticmethod(get_os_type_label)
+
+    def get_os_variant_label(type, variant):
+        return Guest._OS_TYPES[type]["variants"][variant]["label"]
+    get_os_variant_label = staticmethod(get_os_variant_label)
+
+    def __init__(self, type=None, connection=None, hypervisorURI=None,
+                 installer=None):
+        # We specifically ignore the 'type' parameter here, since
+        # it has been replaced by installer.type, and child classes can
+        # use it when creating a default installer.
         self._installer = installer
         self._name = None
         self._uuid = None
@@ -380,6 +527,9 @@ class Guest(object):
         self._cpuset = None
         self._graphics_dev = None
 
+        self._os_type = None
+        self._os_variant = None
+
         # Public device lists unaltered by install process
         self.disks = []
         self.nics = []
@@ -412,7 +562,7 @@ class Guest(object):
     def get_type(self):
         return self._installer.type
     def set_type(self, val):
-        self._installer.type = type
+        self._installer.type = val
     type = property(get_type, set_type)
 
 
@@ -456,19 +606,7 @@ class Guest(object):
     def get_uuid(self):
         return self._uuid
     def set_uuid(self, val):
-        # need better validation
-        if type(val) is not type("string"):
-            raise ValueError, _("UUID must be a string.")
-
-        form = re.match("[a-fA-F0-9]{8}[-]([a-fA-F0-9]{4}[-]){3}[a-fA-F0-9]{12}$", val)
-        if form is None:
-            form = re.match("[a-fA-F0-9]{32}$", val)
-            if form is None:
-                raise ValueError, _("UUID must be a 32-digit hexadecimal number. It may take the form XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX or may omit hyphens altogether.")
-
-            else:   # UUID had no dashes, so add them in
-                val=val[0:8] + "-" + val[8:12] + "-" + val[12:16] + \
-                    "-" + val[16:20] + "-" + val[20:32]
+        val = _util.validate_uuid(val)
         self._uuid = val
     uuid = property(get_uuid, set_uuid)
 
@@ -477,7 +615,7 @@ class Guest(object):
     def get_vcpus(self):
         return self._vcpus
     def set_vcpus(self, val):
-        maxvcpus = util.get_max_vcpus(self.conn, self.type)
+        maxvcpus = _util.get_max_vcpus(self.conn, self.type)
         if type(val) is not int or val < 1:
             raise ValueError, _("Number of vcpus must be a postive integer.")
         if val > maxvcpus:
@@ -495,7 +633,7 @@ class Guest(object):
         if re.match("^[0-9,-]*$", val) is None:
             raise ValueError, _("cpuset can only contain numeric, ',', or '-' characters")
 
-        pcpus = util.get_phy_cpus(self.conn)
+        pcpus = _util.get_phy_cpus(self.conn)
         for c in val.split(','):
             if c.find('-') != -1:
                 (x, y) = c.split('-')
@@ -515,6 +653,47 @@ class Guest(object):
         self._graphics_dev = val
     graphics_dev = property(get_graphics_dev, set_graphics_dev)
 
+    def get_os_type(self):
+        return self._os_type
+    def set_os_type(self, val):
+        if type(val) is not str:
+            raise ValueError(_("OS type must be a string."))
+        val = val.lower()
+        if self._OS_TYPES.has_key(val):
+            self._os_type = val
+            # Invalidate variant, since it may not apply to the new os type
+            self._os_variant = None
+        else:
+            raise ValueError, _("OS type '%s' does not exist in our "
+                                "dictionary") % val
+    os_type = property(get_os_type, set_os_type)
+
+    def get_os_variant(self):
+        return self._os_variant
+    def set_os_variant(self, val):
+        if type(val) is not str:
+            raise ValueError(_("OS variant must be a string."))
+        val = val.lower()
+        if self.os_type:
+            if self._OS_TYPES[self.os_type]["variants"].has_key(val):
+                self._os_variant = val
+            else:
+                raise ValueError, _("OS variant '%(var)s; does not exist in "
+                                    "our dictionary for OS type '%(ty)s'" ) % \
+                                    {'var' : val, 'ty' : self._os_type}
+        else:
+            for ostype in self.list_os_types():
+                if self._OS_TYPES[ostype]["variants"].has_key(val) and \
+                   not self._OS_TYPES[ostype]["variants"][val].get("skip"):
+                    logging.debug("Setting os type to '%s' for variant '%s'" %\
+                                  (ostype, val))
+                    self.os_type = ostype
+                    self._os_variant = val
+                    return
+            raise ValueError, _("Unknown OS variant '%s'" % val)
+    os_variant = property(get_os_variant, set_os_variant)
+
+
     # DEPRECATED PROPERTIES
     # Deprecated: Should set graphics_dev.keymap directly
     def get_keymap(self):
@@ -529,7 +708,7 @@ class Guest(object):
     # Deprecated: Should set guest.graphics_dev = VirtualGraphics(...)
     def get_graphics(self):
         if self._graphics_dev is None:
-            return { "enabled " : False }
+            return { "enabled" : False }
         return { "enabled" : True, "type" : self._graphics_dev, \
                  "keymap"  : self._graphics_dev.keymap}
     def set_graphics(self, val):
@@ -553,10 +732,14 @@ class Guest(object):
                 if val.has_key("opts"):
                     port = val["opts"]
         elif type(val) == tuple:
-            if len(val) >= 1: enabled = val[0]
-            if len(val) >= 2: gtype = val[1]
-            if len(val) >= 3: port = val[2]
-            if len(val) >= 4: keymap = val[3]
+            if len(val) >= 1:
+                enabled = val[0]
+            if len(val) >= 2:
+                gtype = val[1]
+            if len(val) >= 3:
+                port = val[2]
+            if len(val) >= 4:
+                keymap = val[3]
         else:
             if val in ("vnc", "sdl"):
                 gtype = val
@@ -620,7 +803,6 @@ class Guest(object):
     cdrom = property(get_cdrom, set_cdrom)
     # END DEPRECATED PROPERTIES
 
-
     def _create_devices(self,progresscb):
         """Ensure that devices are setup"""
         for disk in self._install_disks:
@@ -628,7 +810,11 @@ class Guest(object):
         for nic in self._install_nics:
             nic.setup(self.conn)
 
-    def _get_network_xml(self, install = True):
+    def _get_disk_xml(self, install=True):
+        """Return xml for disk devices (Must be implemented in subclass)"""
+        raise NotImplementedError
+
+    def _get_network_xml(self):
         """Get the network config in the libvirt XML format"""
         ret = ""
         for n in self._install_nics:
@@ -637,16 +823,21 @@ class Guest(object):
             ret += n.get_xml_config()
         return ret
 
-    def _get_graphics_xml(self, install = True):
+    def _get_graphics_xml(self):
         """Get the graphics config in the libvirt XML format."""
         if self._graphics_dev is None:
             return ""
         return self._graphics_dev.get_xml_config()
 
-    def _get_input_xml(self, install = True):
+    def _get_input_device(self):
+        """ Return a tuple of the form (devtype, bus) for the desired
+            input device. (Must be implemented in subclass) """
+        raise NotImplementedError
+
+    def _get_input_xml(self):
         """Get the input device config in libvirt XML format."""
-        (type,bus) = self.get_input_device()
-        return "    <input type='%s' bus='%s'/>" % (type, bus)
+        (devtype, bus) = self._get_input_device()
+        return "    <input type='%s' bus='%s'/>" % (devtype, bus)
 
     def _get_sound_xml(self):
         """Get the sound device configuration in libvirt XML format."""
@@ -657,13 +848,13 @@ class Guest(object):
             xml += sound_dev.get_xml_config()
         return xml
 
-    def _get_device_xml(self, install = True):
+    def _get_device_xml(self, install=True):
 
         xml = ""
         diskxml     = self._get_disk_xml(install)
-        netxml      = self._get_network_xml(install)
-        inputxml    = self._get_input_xml(install)
-        graphicsxml = self._get_graphics_xml(install)
+        netxml      = self._get_network_xml()
+        inputxml    = self._get_input_xml()
+        graphicsxml = self._get_graphics_xml()
         soundxml    = self._get_sound_xml()
         for devxml in [diskxml, netxml, inputxml, graphicsxml, soundxml]:
             if devxml:
@@ -672,6 +863,9 @@ class Guest(object):
                 xml += devxml
         return xml
 
+    def _get_osblob(self, install):
+        """Return os, features, and clock xml (Implemented in subclass)"""
+        raise NotImplementedError
 
     def get_config_xml(self, install = True, disk_boot = False):
         if install:
@@ -738,12 +932,17 @@ class Guest(object):
         self._install_nics = self.nics[:]
         self._set_defaults()
 
+        self._installer.prepare(guest = self,
+                                meter = meter)
+        if self._installer.install_disk is not None:
+            self._install_disks.append(self._installer.install_disk)
+
     def _do_install(self, consolecb, meter, removeOld=False, wait=True):
         vm = None
         try:
             vm = self.conn.lookupByName(self.name)
         except libvirt.libvirtError:
-           pass
+            pass
 
         if vm is not None:
             if removeOld :
@@ -762,7 +961,7 @@ class Guest(object):
         self._create_devices(meter)
         install_xml = self.get_config_xml()
         if install_xml:
-            logging.debug("Creating guest from '%s'" % ( install_xml ))
+            logging.debug("Creating guest from:\n%s" % install_xml)
             meter.start(size=None, text=_("Creating domain..."))
             self.domain = self.conn.createLinux(install_xml, 0)
             if self.domain is None:
@@ -770,20 +969,8 @@ class Guest(object):
             meter.end(0)
 
             logging.debug("Created guest, looking to see if it is running")
-            # sleep in .25 second increments until either a) we find
-            # our domain or b) it's been 5 seconds.  this is so that
-            # we can try to gracefully handle domain creation failures
-            num = 0
-            d = None
-            while num < (5 / .25): # 5 seconds, .25 second sleeps
-                try:
-                    d = self.conn.lookupByName(self.name)
-                    break
-                except libvirt.libvirtError, e:
-                    logging.debug("No guest running yet " + str(e))
-                    pass
-                num += 1
-                time.sleep(0.25)
+
+            d = _wait_for_domain(self.conn, self.name)
 
             if d is None:
                 raise RuntimeError, _("It appears that your installation has crashed.  You should be able to find more information in the logs")
@@ -793,14 +980,14 @@ class Guest(object):
                 child = consolecb(self.domain)
 
         boot_xml = self.get_config_xml(install = False)
-        logging.debug("Saving XML boot config '%s'" % ( boot_xml ))
+        logging.debug("Saving XML boot config:\n%s" % boot_xml)
         self.conn.defineXML(boot_xml)
 
         if child and wait: # if we connected the console, wait for it to finish
             try:
-                (pid, status) = os.waitpid(child, 0)
-            except OSError, (errno, msg):
-                print __name__, "waitpid:", msg
+                os.waitpid(child, 0)
+            except OSError, (err_no, msg):
+                print __name__, "waitpid: %s: %s" % (err_no, msg)
 
             # ensure there's time for the domain to finish destroying if the
             # install has finished or the guest crashed
@@ -815,21 +1002,8 @@ class Guest(object):
 
     def connect_console(self, consolecb, wait=True):
         logging.debug("Restarted guest, looking to see if it is running")
-        # sleep in .25 second increments until either a) we get running
-        # domain ID or b) it's been 5 seconds.  this is so that
-        # we can try to gracefully handle domain restarting failures
-        num = 0
-        while num < (5 / .25): # 5 seconds, .25 second sleeps
-            try:
-                self.domain = self.conn.lookupByName(self.name)
-                if self.domain and self.domain.ID() != -1:
-                    break
-            except libvirt.libvirtError, e:
-                logging.debug("No guest existing " + str(e))
-                self.domain = None
-                pass
-            num += 1
-            time.sleep(0.25)
+
+        self.domain = _wait_for_domain(self.conn, self.name)
 
         if self.domain is None:
             raise RuntimeError, _("Domain has not existed.  You should be able to find more information in the logs")
@@ -843,9 +1017,10 @@ class Guest(object):
 
         if child and wait: # if we connected the console, wait for it to finish
             try:
-                (pid, status) = os.waitpid(child, 0)
-            except OSError, (errno, msg):
-                raise RuntimeError, "waiting console pid error: %s" % msg
+                os.waitpid(child, 0)
+            except OSError, (err_no, msg):
+                raise RuntimeError, \
+                      "waiting console pid error: %s: %s" % (err_no, msg)
 
     def validate_parms(self):
         if self.domain is not None:
@@ -854,30 +1029,71 @@ class Guest(object):
     def _set_defaults(self):
         if self.uuid is None:
             while 1:
-                self.uuid = util.uuidToString(util.randomUUID())
-                try:
-                    if self.conn.lookupByUUIDString(self.uuid) is not None:
-                        continue
-                    else:
-                        # libvirt probably shouldn't throw an error on a 
-                        # non-matching UUID, so do the right thing on a 
-                        # None return value with no error
-                        break
-                except libvirt.libvirtError:
-                    break
+                self.uuid = _util.uuidToString(_util.randomUUID())
+                if _util.vm_uuid_collision(self.conn, self.uuid):
+                    continue
+                break
         else:
-            try:
-                if self.conn.lookupByUUIDString(self.uuid) is not None:
-                    raise RuntimeError, _("The UUID you entered is already in use by another guest!")
-                else:
-                    pass
-            except libvirt.libvirtError:
-                pass
+            if _util.vm_uuid_collision(self.conn, self.uuid):
+                raise RuntimeError, _("The UUID you entered is already in "
+                                      "use by another guest!")
         if self.vcpus is None:
             self.vcpus = 1
         if self.name is None or self.memory is None:
             raise RuntimeError, _("Name and memory must be specified for all guests!")
 
+    # Guest Dictionary Helper methods
+
+    def _lookup_osdict_key(self, key):
+        """
+        Using self.os_type and self.os_variant to find key in OSTYPES
+        @returns: dict value, or None if os_type/variant wasn't set
+        """
+        typ = self.os_type
+        var = self.os_variant
+        if typ:
+            if var and self._OS_TYPES[typ]["variants"][var].has_key(key):
+                return self._OS_TYPES[typ]["variants"][var][key]
+            elif self._OS_TYPES[typ].has_key(key):
+                return self._OS_TYPES[typ][key]
+        return self._DEFAULTS[key]
+
+    def _lookup_device_param(self, device_key, param):
+        """
+        Check the OS dictionary for the prefered device setting for passed
+        device type and param (bus, model, etc.)
+        """
+        os_devs = self._lookup_osdict_key("devices")
+        default_devs = self._DEFAULTS["devices"]
+        for devs in [os_devs, default_devs]:
+            if not devs.has_key(device_key):
+                continue
+            for ent in devs[device_key][param]:
+                hv_types = ent[0]
+                param_value = ent[1]
+                if self.type in hv_types:
+                    return param_value
+                elif "all" in hv_types:
+                    return param_value
+        raise RuntimeError(_("Invalid dictionary entry for device '%s %s'" % \
+                             (device_key, param)))
+
+def _wait_for_domain(conn, name):
+    # sleep in .25 second increments until either a) we get running
+    # domain ID or b) it's been 5 seconds.  this is so that
+    # we can try to gracefully handle domain restarting failures
+    dom = None
+    for ignore in range(1, (5 / .25)): # 5 seconds, .25 second sleeps
+        try:
+            dom = conn.lookupByName(name)
+            if dom and dom.ID() != -1:
+                break
+        except libvirt.libvirtError, e:
+            logging.debug("No guest running yet: " + str(e))
+            dom = None
+        time.sleep(0.25)
+
+    return dom
+
 # Back compat class to avoid ABI break
-class XenGuest(Guest):
-	pass
+XenGuest = Guest
