@@ -25,32 +25,102 @@ import gzip
 import re
 import tempfile
 import platform
+import socket
 import ConfigParser
 
+from virtinst import _util
 from virtinst import _virtinst as _
 
+def distroFromTreeinfo(fetcher, progresscb, uri, vmtype=None,
+                       scratchdir=None, arch=None):
+    # Parse treeinfo 'family' field, and return the associated Distro class
+    # None if no treeinfo, GenericDistro if unknown family type.
+    if not fetcher.hasFile(".treeinfo"):
+        return None
+
+    tmptreeinfo = fetcher.acquireFile(".treeinfo", progresscb)
+    try:
+        treeinfo = ConfigParser.SafeConfigParser()
+        treeinfo.read(tmptreeinfo)
+    finally:
+        os.unlink(tmptreeinfo)
+
+    fam = treeinfo.get("general", "family")
+
+    if re.match(".*Fedora.*", fam):
+        ob = FedoraDistro(uri, vmtype, scratchdir, arch)
+    elif re.match(".*CentOS.*", fam):
+        ob = CentOSDistro(uri, vmtype, scratchdir, arch)
+    elif re.match(".*Red Hat Enterprise Linux.*", fam):
+        ob = RHELDistro(uri, vmtype, scratchdir, arch)
+    elif re.match(".*Scientific Linux.*", fam):
+        ob = SLDistro(uri, vmtype, scratchdir, arch)
+    else:
+        ob = GenericDistro(uri, vmtype, scratchdir, arch)
+
+    ob.treeinfo = treeinfo
+    return ob
+
+
 # An image store is a base class for retrieving either a bootable
 # ISO image, or a kernel+initrd  pair for a particular OS distribution
 class Distro:
 
-    def __init__(self, uri, type=None, scratchdir=None, arch=None):
+    name = ""
+    _boot_iso_paths = []
+    _hvm_kernel_paths = []
+    _xen_kernel_paths = []
+    uses_treeinfo = False
+
+    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
         self.uri = uri
-        self.type = type
+        self.type = vmtype
         self.scratchdir = scratchdir
         if arch == None:
             arch = platform.machine()
         self.arch = arch
         self.treeinfo = None
 
-    def acquireBootDisk(self, fetcher, progresscb):
-        raise "Not implemented"
-
-    def acquireKernel(self, fetcher, progresscb):
-        raise "Not implemented"
-
     def isValidStore(self, fetcher, progresscb):
         """Determine if uri points to a tree of the store's distro"""
-        raise "Not implemented"
+        raise NotImplementedError
+
+    def acquireKernel(self, guest, fetcher, progresscb):
+        kernelpath = None
+        initrdpath = None
+        if self._hasTreeinfo(fetcher, progresscb):
+            kernelpath = self._getTreeinfoMedia("kernel")
+            initrdpath = self._getTreeinfoMedia("initrd")
+        else:
+            # fall back to old code
+            if self.type is None or self.type == "hvm":
+                paths = self._hvm_kernel_paths
+            else:
+                paths = self._xen_kernel_paths
+
+            for kpath, ipath in paths:
+                if fetcher.hasFile(kpath) and fetcher.hasFile(ipath):
+                    kernelpath = kpath
+                    initrdpath = ipath
+
+        if not kernelpath or not initrdpath:
+            raise RuntimeError(_("Couldn't find %(type)s kernel for "
+                                 "%(distro)s tree.") % \
+                                 { "distro": self.name, "type" : self.type })
+
+        return self._kernelFetchHelper(fetcher, guest, progresscb, kernelpath,
+                                       initrdpath)
+
+    def acquireBootDisk(self, fetcher, progresscb):
+        if self._hasTreeinfo(fetcher, progresscb):
+            return fetcher.acquireFile(self._getTreeinfoMedia("boot.iso"))
+        else:
+            for path in self._boot_iso_paths:
+                if fetcher.hasFile(path):
+                    return fetcher.acquireFile(path, progresscb)
+            raise RuntimeError(_("Could not find boot.iso in %s tree." % \
+                               self.name))
+
 
     def _hasTreeinfo(self, fetcher, progresscb):
         # all Red Hat based distros should have .treeinfo, perhaps others
@@ -58,7 +128,7 @@ class Distro:
         if not (self.treeinfo is None):
             return True
 
-        if not fetcher.hasFile(".treeinfo"):
+        if not self.uses_treeinfo or not fetcher.hasFile(".treeinfo"):
             return False
 
         logging.debug("Detected .treeinfo file")
@@ -71,18 +141,66 @@ class Distro:
             os.unlink(tmptreeinfo)
         return True
 
-    def _getTreeinfoMedia(self, fetcher, progresscb, mediaName):
+    def _getTreeinfoMedia(self, mediaName):
         if self.type == "xen":
-            type = "xen"
+            t = "xen"
         else:
-            type = self.treeinfo.get("general", "arch")
+            t = self.treeinfo.get("general", "arch")
 
-        return self.treeinfo.get("images-%s" % type, mediaName)
+        return self.treeinfo.get("images-%s" % t, mediaName)
+
+    def _fetchAndMatchRegex(self, fetcher, progresscb, filename, regex):
+        # Fetch 'filename' and return True/False if it matches the regex
+        local_file = None
+        try:
+            try:
+                local_file = fetcher.acquireFile(filename, progresscb)
+            except:
+                return False
+
+            f = open(local_file, "r")
+            try:
+                while 1:
+                    buf = f.readline()
+                    if not buf:
+                        break
+                    if re.match(regex, buf):
+                        return True
+            finally:
+                f.close()
+        finally:
+            if local_file is not None:
+                os.unlink(local_file)
+
+        return False
+
+    def _kernelFetchHelper(self, fetcher, guest, progresscb, kernelpath, initrdpath):
+        # Simple helper for fetching kernel + initrd and performing
+        # cleanup if neccessary
+        kernel = fetcher.acquireFile(kernelpath, progresscb)
+        args = ''
+
+        if not fetcher.location.startswith("/"):
+            args += "method=" + fetcher.location
+
+        if guest.extraargs:
+            args += guest.extraargs
+
+        try:
+            initrd = fetcher.acquireFile(initrdpath, progresscb)
+            return kernel, initrd, args
+        except:
+            os.unlink(kernel)
 
 
 class GenericDistro(Distro):
     """Generic distro store. Check well known paths for kernel locations
        as a last resort if we can't recognize any actual distro"""
+
+    name = "Generic"
+    os_type = "linux"
+    uses_treeinfo = True
+
     _xen_paths = [ ("images/xen/vmlinuz",
                     "images/xen/initrd.img"),           # Fedora
                  ]
@@ -103,14 +221,15 @@ class GenericDistro(Distro):
         if self._hasTreeinfo(fetcher, progresscb):
             # Use treeinfo to pull down media paths
             if self.type == "xen":
-                type = "xen"
+                typ = "xen"
             else:
-                type = self.treeinfo.get("general", "arch")
-            kernelSection = "images-%s" % type
+                typ = self.treeinfo.get("general", "arch")
+            kernelSection = "images-%s" % typ
             isoSection = "images-%s" % self.treeinfo.get("general", "arch")
 
             if self.treeinfo.has_section(kernelSection):
-                self._valid_kernel_path = (self._getTreeinfoMedia(fetcher, progresscb, "kernel"), self._getTreeinfoMedia(fetcher, progresscb, "initrd"))
+                self._valid_kernel_path = (self._getTreeinfoMedia("kernel"),
+                                           self._getTreeinfoMedia("initrd"))
             if self.treeinfo.has_section(isoSection):
                 self._valid_iso_path = self.treeinfo.get(isoSection, "boot.iso")
 
@@ -136,20 +255,14 @@ class GenericDistro(Distro):
             return True
         return False
 
-    def acquireKernel(self, fetcher, progresscb):
+    def acquireKernel(self, guest, fetcher, progresscb):
         if self._valid_kernel_path == None:
             raise ValueError(_("Could not find a kernel path for virt type "
                                "'%s'" % self.type))
 
-        kernel = fetcher.acquireFile(self._valid_kernel_path[0], progresscb)
-        try:
-            initrd = fetcher.acquireFile(self._valid_kernel_path[1], progresscb)
-            if fetcher.location.startswith("/"):
-                return (kernel, initrd, "")
-            else:
-                return (kernel, initrd, "method=" + fetcher.location)
-        except:
-            os.unlink(kernel)
+        return self._kernelFetchHelper(fetcher, guest, progresscb,
+                                       self._valid_kernel_path[0],
+                                       self._valid_kernel_path[1])
 
     def acquireBootDisk(self, fetcher, progresscb):
         if self._valid_iso_path == None:
@@ -162,40 +275,25 @@ class GenericDistro(Distro):
 # a common layout
 class RedHatDistro(Distro):
 
-    def acquireKernel(self, fetcher, progresscb):
-        if self._hasTreeinfo(fetcher, progresscb):
-            kernelpath = self._getTreeinfoMedia(fetcher, progresscb, "kernel")
-            initrdpath = self._getTreeinfoMedia(fetcher, progresscb, "initrd")
-        else:
-            # fall back to old code
-            if self.type is None or self.type == "hvm":
-                kernelpath = "images/pxeboot/vmlinuz"
-                initrdpath = "images/pxeboot/initrd.img"
-            else:
-                kernelpath = "images/%s/vmlinuz" % (self.type)
-                initrdpath = "images/%s/initrd.img" % (self.type)
+    name = "Red Hat"
+    os_type = "linux"
+    uses_treeinfo = True
+    _boot_iso_paths   = [ "images/boot.iso" ]
+    _hvm_kernel_paths = [ ("images/pxeboot/vmlinuz",
+                           "images/pxeboot/initrd.img") ]
+    _xen_kernel_paths = [ ("images/xen/vmlinuz",
+                           "images/xen/initrd.img") ]
 
-        kernel = fetcher.acquireFile(kernelpath, progresscb)
-        try:
-            initrd = fetcher.acquireFile(initrdpath, progresscb)
-            if fetcher.location.startswith("/"):
-                # Local host path, so can't pass a location to guest for install method
-                return (kernel, initrd, "")
-            else:
-                return (kernel, initrd, "method=" + fetcher.location)
-        except:
-            os.unlink(kernel)
+    def isValidStore(self, fetcher, progresscb):
+        raise NotImplementedError
 
-    def acquireBootDisk(self, fetcher, progresscb):
-        if self._hasTreeinfo(fetcher, progresscb):
-            return fetcher.acquireFile(self._getTreeinfoMedia(fetcher,
-                                                              progresscb,
-                                                              "boot.iso"))
-        else:
-            return fetcher.acquireFile("images/boot.iso", progresscb)
 
 # Fedora distro check
 class FedoraDistro(RedHatDistro):
+
+    name = "Fedora"
+    os_type = "linux"
+
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Fedora.*", self.treeinfo.get("general", "family"))
@@ -208,6 +306,10 @@ class FedoraDistro(RedHatDistro):
 
 # Red Hat Enterprise Linux distro check
 class RHELDistro(RedHatDistro):
+
+    name = "Red Hat Enterprise Linux"
+    os_type = "linux"
+
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*Red Hat Enterprise Linux.*", self.treeinfo.get("general", "family"))
@@ -227,6 +329,10 @@ class RHELDistro(RedHatDistro):
 
 # CentOS distro check
 class CentOSDistro(RedHatDistro):
+
+    name = "CentOS"
+    os_type = "linux"
+
     def isValidStore(self, fetcher, progresscb):
         if self._hasTreeinfo(fetcher, progresscb):
             m = re.match(".*CentOS.*", self.treeinfo.get("general", "family"))
@@ -240,48 +346,62 @@ class CentOSDistro(RedHatDistro):
 
 # Scientific Linux distro check
 class SLDistro(RedHatDistro):
+
+    name = "Scientific Linux"
+    os_type = "linux"
+    _boot_iso_paths = RedHatDistro._boot_iso_paths + [ "images/SL/boot.iso" ]
+    _hvm_kernel_paths = RedHatDistro._hvm_kernel_paths + \
+                        [ ("images/SL/pxeboot/vmlinuz",
+                           "images/SL/pxeboot/initrd.img") ]
+
     def isValidStore(self, fetcher, progresscb):
-        if fetcher.hasFile("SL"):
-            logging.debug("Detected a Scientific Linux distro")
-            return True
-        return False
+        if self._hasTreeinfo(fetcher, progresscb):
+            m = re.match(".*Scientific Linux.*",
+                         self.treeinfo.get("general", "family"))
+            return (m != None)
+        else:
+            if fetcher.hasFile("SL"):
+                logging.debug("Detected a Scientific Linux distro")
+                return True
+            return False
 
 
 
 # Suse  image store is harder - we fetch the kernel RPM and a helper
 # RPM and then munge bits together to generate a initrd
 class SuseDistro(Distro):
-    def __init__(self, uri, type=None, scratchdir=None, arch=None):
-        Distro.__init__(self, uri, type, scratchdir, arch)
-        if len(self.arch) == 4 and self.arch[0] == 'i' \
-           and self.arch[2:] == "86":
-            self.arch = "i386"
 
-    def acquireBootDisk(self, fetcher, progresscb):
-        return fetcher.acquireFile("boot/boot.iso", progresscb)
+    name = "SUSE"
+    os_type = "linux"
+    _boot_iso_paths   = [ "boot/boot.iso" ]
+    _hvm_kernel_paths = []
+    _xen_kernel_paths = []
 
-    def acquireKernel(self, fetcher, progresscb):
-        kernel_args = ""
-        if not fetcher.location.startswith("/"):
-            kernel_args = "method=" + fetcher.location
+    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
+        Distro.__init__(self, uri, vmtype, scratchdir, arch)
+        if re.match(r'i[4-9]86', arch):
+            self.arch = 'i386'
 
+        # Tested with Opensuse 10, 11, and sles 10
+        self._hvm_kernel_paths += [ ("boot/%s/loader/linux" % self.arch,
+                                     "boot/%s/loader/initrd" % self.arch) ]
+        # Matches Opensuse > 10.2 and sles 10
+        self._xen_kernel_paths += [ ("boot/%s/vmlinuz-xen" % self.arch,
+                                     "boot/%s/initrd-xen" % self.arch) ]
+
+    def isValidStore(self, fetcher, progresscb):
+        # Suse distros always have a 'directory.yast' file in the top
+        # level of install tree, which we use as the magic check
+        if fetcher.hasFile("directory.yast"):
+            logging.debug("Detected a Suse distro.")
+            return True
+        return False
+
+    def acquireKernel(self, guest, fetcher, progresscb):
         # If installing a fullvirt guest
-        if self.type is None or self.type == "hvm":
-            # Tested with Opensuse 10, 11, and sles 10
-            kernel = fetcher.acquireFile("boot/%s/loader/linux" % self.arch,
-                                         progresscb)
-            initrd = fetcher.acquireFile("boot/%s/loader/initrd" % self.arch,
-                                         progresscb)
-            return (kernel, initrd, kernel_args)
-
-        # Else we are looking for a paravirt kernel
-        if fetcher.hasFile("boot/%s/vmlinuz-xen" % self.arch):
-            # Should match opensuse > 10.2 and sles 10
-            kernel = fetcher.acquireFile("boot/%s/vmlinuz-xen" % self.arch,
-                                         progresscb)
-            initrd = fetcher.acquireFile("boot/%s/initrd-xen" % self.arch,
-                                         progresscb)
-            return (kernel, initrd, kernel_args)
+        if self.type is None or self.type == "hvm" or \
+           fetcher.hasFile("boot/%s/vmlinuz-xen" % self.arch):
+            return Distro.acquireKernel(self, guest, fetcher, progresscb)
 
         # For Opensuse <= 10.2, we need to perform some heinous stuff
         logging.debug("Trying Opensuse 10 PV rpm hacking")
@@ -333,28 +453,28 @@ class SuseDistro(Distro):
 
             installinitrdrpm = None
             kernelrpm = None
-            dir = None
+            dirname = None
             while 1:
                 data = filelistData.readline()
                 if not data:
                     break
-                if dir is None:
+                if dirname is None:
                     for arch in arches:
                         wantdir = "/suse/" + arch
                         if data == "." + wantdir + ":\n":
-                            dir = wantdir
+                            dirname = wantdir
                             break
                 else:
                     if data == "\n":
-                        dir = None
+                        dirname = None
                     else:
                         if data[:5] != "total":
                             filename = re.split("\s+", data)[8]
 
                             if filename[:14] == "install-initrd":
-                                installinitrdrpm = dir + "/" + filename
+                                installinitrdrpm = dirname + "/" + filename
                             elif filename[:len(kernelname)] == kernelname:
-                                kernelrpm = dir + "/" + filename
+                                kernelrpm = dirname + "/" + filename
 
             if kernelrpm is None:
                 raise Exception(_("Unable to determine kernel RPM path"))
@@ -392,7 +512,7 @@ class SuseDistro(Distro):
 
             # Build a list of all .ko files
             modpaths = {}
-            for root, dirs, files in os.walk(cpiodir + "/kernel/lib/modules", topdown=False):
+            for root, dummy, files in os.walk(cpiodir + "/kernel/lib/modules", topdown=False):
                 for name in files:
                     if name.endswith(".ko"):
                         modpaths[name] = os.path.join(root, name)
@@ -477,71 +597,101 @@ class SuseDistro(Distro):
             os.system("rm -rf " + cpiodir)
 
 
-    def isValidStore(self, fetcher, progresscb):
-        # Suse distros always have a 'directory.yast' file in the top
-        # level of install tree, which we use as the magic check
-        if fetcher.hasFile("directory.yast"):
-            logging.debug("Detected a Suse distro.")
-            return True
-        return False
-
-
 class DebianDistro(Distro):
+    # ex. http://ftp.egr.msu.edu/debian/dists/sarge/main/installer-i386/
+    # daily builds: http://people.debian.org/~joeyh/d-i/
+
+    name = "Debian"
+    os_type = "linux"
+
+    def __init__(self, uri, vmtype=None, scratchdir=None, arch=None):
+        Distro.__init__(self, uri, vmtype, scratchdir, arch)
+        if uri.count("installer-i386"):
+            self._treeArch = "i386"
+        elif uri.count("installer-amd64"):
+            self._treeArch = "amd64"
+        else:
+            self._treeArch = "i386"
+
+        if re.match(r'i[4-9]86', arch):
+            self.arch = 'i386'
+
+        self._prefix = 'current/images'
+        self._set_media_paths()
+
+    def _set_media_paths(self):
+        # Use self._prefix to set media paths
+        self._boot_iso_paths   = [ "%s/netboot/mini.iso" % self._prefix ]
+        hvmroot = "%s/netboot/debian-installer/%s/" % (self._prefix,
+                                                       self._treeArch)
+        xenroot = "%s/netboot/xen/" % self._prefix
+        self._hvm_kernel_paths = [ (hvmroot + "linux", hvmroot + "initrd.gz") ]
+        self._xen_kernel_paths = [ (xenroot + "vmlinuz",
+                                    xenroot + "initrd.gz") ]
+
+    def isValidStore(self, fetcher, progresscb):
+
+        # For regular trees
+        if fetcher.hasFile("%s/MANIFEST" % self._prefix):
+            pass
+        # For daily trees
+        elif fetcher.hasFile("images/daily/MANIFEST"):
+            self._prefix = "images/daily"
+            self._set_media_paths()
+        else:
+            logging.debug("Doesn't look like a Debian distro.")
+            return False
+
+        filename = "%s/MANIFEST" % self._prefix
+
+        if self._fetchAndMatchRegex(fetcher, progresscb, filename,
+                                    ".*debian-installer.*"):
+            logging.debug("Detected a Debian distro")
+            return True
+
+        return False
+
+
+class UbuntuDistro(DebianDistro):
+
+    name = "Ubuntu"
+    os_type = "linux"
+
+    def _set_media_paths(self):
+        DebianDistro._set_media_paths(self)
+        root = "%s/netboot/ubuntu-installer/%s/" % (self._prefix,
+                                                    self._treeArch)
+        self._hvm_kernel_paths = [ (root + "linux", root + "initrd.gz") ]
+        self._xen_kernel_paths = []
+
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
         if self.type is not None and self.type != "hvm":
             return False
 
-        file = None
-        try:
-            try:
-                file = None
-                if fetcher.hasFile("current/images/MANIFEST"):
-                    file = fetcher.acquireFile("current/images/MANIFEST", 
-                                               progresscb)
-                else:
-                    logging.debug("Doesn't look like a Debian distro.")
-                    return False
-                    
-            except ValueError, e:
-                logging.debug("Doesn't look like a Debian distro " + str(e))
-                return False
-            f = open(file, "r")
-            try:
-                while 1:
-                    buf = f.readline()
-                    if not buf:
-                        break
-                    if re.match(".*debian.*", buf):
-                        logging.debug("Detected a Debian distro")
-                        return True
-            finally:
-                f.close()
-        finally:
-            if file is not None:
-                os.unlink(file)
-        return False
-
-    def acquireBootDisk(self, fetcher, progresscb):
-        # eg from http://ftp.egr.msu.edu/debian/dists/sarge/main/installer-i386/
-        return fetcher.acquireFile("current/images/netboot/mini.iso", progresscb)
-
-
-class UbuntuDistro(Distro):
-    def isValidStore(self, fetcher, progresscb):
-        # Don't support any paravirt installs
-        if self.type is not None and self.type != "hvm":
+        # For regular trees
+        if not fetcher.hasFile("%s/MANIFEST" % self._prefix):
             return False
+
+        if self._fetchAndMatchRegex(fetcher, progresscb,
+                                    "%s/MANIFEST" % self._prefix,
+                                    ".*ubuntu-installer.*"):
+            logging.debug("Detected an Ubuntu distro")
+            return True
+
         return False
 
-class GentooDistro(Distro):
-    def isValidStore(self, fetcher, progresscb):
-        # Don't support any paravirt installs
-        if self.type is not None and self.type != "hvm":
-            return False
-        return False
 
 class MandrivaDistro(Distro):
+    # Ex. ftp://ftp.uwsg.indiana.edu/linux/mandrake/official/2007.1/x86_64/
+
+    name = "Mandriva"
+    os_type = "linux"
+    _boot_iso_paths = [ "install/images/boot.iso" ]
+    # Kernels for HVM: valid for releases 2007.1, 2008.*, 2009.0
+    _hvm_kernel_paths = [ ("isolinux/alt0/vmlinuz", "isolinux/alt0/all.rdz")]
+    _xen_kernel_paths = []
+
     def isValidStore(self, fetcher, progresscb):
         # Don't support any paravirt installs
         if self.type is not None and self.type != "hvm":
@@ -550,26 +700,200 @@ class MandrivaDistro(Distro):
         # Mandriva websites / media appear to have a VERSION
         # file in top level which we can use as our 'magic'
         # check for validity
-        version = None
-        try:
-            try:
-                version = fetcher.acquireFile("VERSION")
-            except:
-                return False
-            f = open(version, "r")
-            try:
-                info = f.readline()
-                if info.startswith("Mandriva"):
-                    logging.debug("Detected a Mandriva distro")
-                    return True
-            finally:
-                f.close()
-        finally:
-            if version is not None:
-                os.unlink(version)
+        if self._fetchAndMatchRegex(fetcher, progresscb, "VERSION",
+                                    ".*Mandriva.*"):
+            logging.debug("Detected a Mandriva distro")
+            return True
 
         return False
 
+# Solaris and OpenSolaris distros
+class SunDistro(Distro):
+
+    name = "Solaris"
+    os_type = "solaris"
+
+    def isValidStore(self, fetcher, progresscb):
+        """Determine if uri points to a tree of the store's distro"""
+        raise NotImplementedError
+
     def acquireBootDisk(self, fetcher, progresscb):
-        #
-        return fetcher.acquireFile("install/images/boot.iso", progresscb)
+        return fetcher.acquireFile("images/solarisdvd.iso", progresscb)
+
+    def process_extra_args(self, argstr):
+        """Collect additional arguments."""
+        if not argstr:
+            return (None, None, None, None)
+
+        kopts = ''
+        kargs = ''
+        smfargs = ''
+        Bargs = ''
+
+        args = argstr.split()
+        i = 0
+        while i < len(args):
+            exarg = args[i]
+            if exarg == '-B':
+                i += 1
+                if i == len(args):
+                    continue
+
+                if not Bargs:
+                    Bargs = args[i]
+                else:
+                    Bargs = ','.join([Bargs, args[i]])
+        
+            elif exarg == '-m':
+                i += 1
+                if i == len(args):
+                    continue
+                smfargs = args[i]
+            elif exarg.startswith('-'):
+                if kopts is None:
+                    kopts = exarg[1:]
+                else:
+                    kopts = kopts + exarg[1:]
+            else:
+                if kargs is None:
+                    kargs = exarg
+                else:
+                    kargs = kargs + ' ' + exarg
+            i += 1
+
+        return kopts, kargs, smfargs, Bargs
+
+class SolarisDistro(SunDistro):
+    kernelpath = 'boot/platform/i86xpv/kernel/unix'
+    initrdpath = 'boot/x86.miniroot'
+
+    def isValidStore(self, fetcher, progresscb):
+        if fetcher.hasFile(self.kernelpath):
+            logging.debug('Detected Solaris')
+            return True
+        return False
+
+    def install_args(self, guest):
+        """Construct kernel cmdline args for the installer, consisting of:
+           the pathname of the kernel (32/64) to load, kernel options
+           and args, and '-B' boot properties."""
+
+        # XXX: ignoring smfargs for the time being
+        (kopts, kargs, smfargs, kbargs) = \
+            self.process_extra_args(guest.extraargs)
+
+        args = [ '' ]
+        if kopts:
+            args += [ '-%s' % kopts ]
+        if kbargs:
+            args += [ '-B', kbargs ]
+
+        netmask = ''
+        # Yuck. Non-default netmasks require this option to be passed.
+        # It's distinctly not-trivial to work out the netmask to be used
+        # automatically.
+        if kargs:
+            for karg in kargs.split():
+                if karg.startswith('subnet-mask'):
+                    netmask = karg.split('=')[1]
+                else:
+                    args += [ kargs ]
+
+        iargs = ''
+        if not guest.graphics['enabled']:
+            iargs += 'nowin '
+
+        if guest.location.startswith('nfs:'):
+            try:
+                guestIP = socket.gethostbyaddr(guest.name)[2][0]
+            except:
+                iargs += ' dhcp'
+            else:
+                iserver = guest.location.split(':')[1]
+                ipath = guest.location.split(':')[2]
+                iserverIP = socket.gethostbyaddr(iserver)[2][0]
+                iargs += ' -B install_media=' + iserverIP + ':' + ipath
+                iargs += ',host-ip=' + guestIP
+                if netmask:
+                    iargs += ',subnet-mask=%s' % netmask
+                droute = _util.default_route(guest.nics[0].bridge)
+                if droute:
+                    iargs += ',router-ip=' + droute
+                if guest.nics[0].macaddr:
+                    en = guest.nics[0].macaddr.split(':')
+                    for i in range(len(en)):
+                        # remove leading '0' from mac address element
+                        if len(en[i]) > 1 and en[i][0] == '0':
+                            en[i] = en[i][1]
+                    boot_mac = ':'.join(en)
+                    iargs += ',boot-mac=' + boot_mac
+        else:
+            iargs += '-B install_media=cdrom'
+
+        args += [ '-', iargs ]
+        return ' '.join(args)
+
+    def acquireKernel(self, guest, fetcher, progresscb):
+
+        try:
+            kernel = fetcher.acquireFile(self.kernelpath, progresscb)
+        except:
+            raise RuntimeError("Solaris PV kernel not found at %s" %
+                self.kernelpath)
+
+        # strip boot from the kernel path
+        kpath = self.kernelpath.split('/')[1:]
+        args = "/" + "/".join(kpath) + self.install_args(guest)
+
+        try:
+            initrd = fetcher.acquireFile(self.initrdpath, progresscb)
+            return (kernel, initrd, args)
+        except:
+            os.unlink(kernel)
+            raise RuntimeError(_("Solaris miniroot not found at %s") %
+                self.initrdpath)
+
+class OpenSolarisDistro(SunDistro):
+    kernelpath = "platform/i86xpv/kernel/unix"
+    initrdpath = "boot/x86.microroot"
+
+    def isValidStore(self, fetcher, progresscb):
+        if fetcher.hasFile(self.kernelpath):
+            logging.debug("Detected OpenSolaris")
+            return True
+        return False
+
+    def install_args(self, guest):
+        """Construct kernel cmdline args for the installer, consisting of:
+           the pathname of the kernel (32/64) to load, kernel options
+           and args, and '-B' boot properties."""
+
+        # XXX: ignoring smfargs and kargs for the time being
+        (kopts, kargs, smfargs, kbargs) = \
+            self.process_extra_args(guest.extraargs)
+
+        args = ''
+        if kopts:
+            args += '-' + kopts
+        if kbargs:
+            args += ' -B ' + kbargs
+
+        return args
+
+    def acquireKernel(self, guest, fetcher, progresscb):
+
+        try:
+            kernel = fetcher.acquireFile(self.kernelpath, progresscb)
+        except:
+            raise RuntimeError(_("OpenSolaris PV kernel not found at %s") %
+                self.kernelpath)
+
+        args = "/" + self.kernelpath + self.install_args(guest)
+
+        try:
+            initrd = fetcher.acquireFile(self.initrdpath, progresscb)
+            return (kernel, initrd, args)
+        except:
+            os.unlink(kernel)
+            raise RuntimeError(_("OpenSolaris microroot not found at %s") %
+                self.initrdpath)
