@@ -50,10 +50,9 @@ import libvirt
 
 import re
 import logging
-import libxml2
 from xml.sax.saxutils import escape
 
-import util
+import _util
 from virtinst import _virtinst as _
 
 DEFAULT_DEV_TARGET = "/dev"
@@ -97,24 +96,17 @@ class StorageObject(object):
 
 
     ## Properties
-    """object_type: pool or volume"""
     def get_object_type(self):
+        # 'pool' or 'volume'
         return self._object_type
     object_type = property(get_object_type)
 
-    def get_type(self):
-        raise RuntimeError, "Must be implemented in child class."
-    type = property(get_type, doc=\
-    """
-    type of the underlying object. could be "dir" for a pool, etc.
-    """)
-
     def get_conn(self):
         return self._conn
     def set_conn(self, val):
         if not isinstance(val, libvirt.virConnect):
             raise ValueError(_("'conn' must be a libvirt connection object."))
-        if not util.is_storage_capable(val):
+        if not _util.is_storage_capable(val):
             raise ValueError(_("Passed connection is not libvirt storage "
                                "capable"))
         self._conn = val
@@ -138,9 +130,7 @@ class StorageObject(object):
         # Check that name doesn't collide with other storage objects
         self._check_name_collision(val)
         self._name = val
-    name = property(get_name, set_name, doc="""
-    Name of the storage object
-    """)
+    name = property(get_name, set_name, doc=_("Name for the storage object."))
 
     # Get/Set methods for use by some objects. Will register where applicable
     def get_perms(self):
@@ -170,13 +160,14 @@ class StorageObject(object):
         raise RuntimeError, "Must be implemented in subclass"
 
     def _get_perms_xml(self):
-        if not self.perms:
+        perms = self.get_perms()
+        if not perms:
             return ""
         return "    <permissions>\n" + \
-               "      <mode>%o</mode>\n" % self.perms["mode"] + \
-               "      <owner>%d</owner>\n" % self.perms["owner"] + \
-               "      <group>%d</group>\n" % self.perms["group"] + \
-               "      <label>%s</label>\n" % self.perms["label"] + \
+               "      <mode>%o</mode>\n" % perms["mode"] + \
+               "      <owner>%d</owner>\n" % perms["owner"] + \
+               "      <group>%d</group>\n" % perms["group"] + \
+               "      <label>%s</label>\n" % perms["label"] + \
                "    </permissions>\n"
 
 
@@ -187,10 +178,11 @@ class StorageObject(object):
         @returns: xml description
         @rtype: C{str}
         """
-        if self.type is None:
+        if not hasattr(self, "type"):
             root_xml = "<%s>\n" % self.object_type
         else:
-            root_xml = "<%s type='%s'>\n" % (self.object_type, self.type)
+            _type = getattr(self, "type")
+            root_xml = "<%s type='%s'>\n" % (self.object_type, _type)
 
         xml = "%s" % (root_xml) + \
               """  <name>%s</name>\n""" % (self.name) + \
@@ -279,19 +271,19 @@ class StoragePool(StorageObject):
         self._source_path = None
         if not uuid:
             self._uuid = None
-        self._random_uuid = util.uuidToString(util.randomUUID())
+        self._random_uuid = _util.uuidToString(_util.randomUUID())
 
     # Properties used by all pools
     def get_type(self):
         return self._type
-    type = property(get_type)
+    type = property(get_type,
+                    doc=_("Storage device type the pool will represent."))
 
     def get_target_path(self):
         return self._target_path
     def set_target_path(self, val):
         self._validate_path(val)
         self._target_path = val
-    target_path = property(get_target_path, set_target_path)
 
     # Get/Set methods for use by some pools. Will be registered when applicable
     def get_source_path(self):
@@ -311,18 +303,7 @@ class StoragePool(StorageObject):
     def get_uuid(self):
         return self._uuid
     def set_uuid(self, val):
-        if type(val) is not type("string"):
-            raise ValueError, _("UUID must be a string.")
-
-        form = re.match("[a-fA-F0-9]{8}[-]([a-fA-F0-9]{4}[-]){3}[a-fA-F0-9]{12}$", val)
-        if form is None:
-            form = re.match("[a-fA-F0-9]{32}$", val)
-            if form is None:
-                raise ValueError, _("UUID must be a 32-digit hexadecimal number. It may take the form XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX or may omit hyphens altogether.")
-
-            else:   # UUID had no dashes, so add them in
-                val=val[0:8] + "-" + val[8:12] + "-" + val[12:16] + \
-                    "-" + val[16:20] + "-" + val[20:32]
+        val = _util.validate_uuid(val)
         self._uuid = val
     uuid = property(get_uuid, set_uuid)
 
@@ -416,6 +397,9 @@ class DirectoryPool(StoragePool):
 
     # Register applicable property methods from parent class
     perms = property(StorageObject.get_perms, StorageObject.set_perms)
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Directory to use for the storage pool."))
 
     def __init__(self, conn, name, target_path=None, uuid=None, perms=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_DIR,
@@ -450,7 +434,11 @@ class FilesystemPool(StoragePool):
     # Register applicable property methods from parent class
     perms = property(StorageObject.get_perms, StorageObject.set_perms)
     source_path = property(StoragePool.get_source_path,
-                           StoragePool.set_source_path)
+                           StoragePool.set_source_path,
+                           doc=_("The existing device to mount for the pool."))
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Location to mount the source device."))
 
     def __init__(self, conn, name, source_path=None, target_path=None,
                  format="auto", uuid=None, perms=None):
@@ -470,7 +458,8 @@ class FilesystemPool(StoragePool):
         if not val in self.formats:
             raise ValueError(_("Unknown Filesystem format: %s" % val))
         self._format = val
-    format = property(get_format, set_format)
+    format = property(get_format, set_format,
+                      doc=_("Filesystem type of the source device."))
 
     def _get_default_target_path(self):
         path = (DEFAULT_DIR_TARGET_BASE + self.name)
@@ -501,13 +490,18 @@ class NetworkFilesystemPool(StoragePool):
 
     # Register applicable property methods from parent class
     source_path = property(StoragePool.get_source_path,
-                           StoragePool.set_source_path)
-    host = property(StoragePool.get_host, StoragePool.set_host)
+                           StoragePool.set_source_path,
+                           doc=_("Path on the host that is being shared."))
+    host = property(StoragePool.get_host, StoragePool.set_host,
+                    doc=_("Name of the host sharing the storage."))
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Location to mount the source device."))
 
     def __init__(self, conn, name, source_path=None, host=None,
                  target_path=None, format="auto", uuid=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_NETFS,
-                             uuid=None, target_path=target_path, conn=conn)
+                             uuid=uuid, target_path=target_path, conn=conn)
 
         self.format = format
 
@@ -522,7 +516,8 @@ class NetworkFilesystemPool(StoragePool):
         if not val in self.formats:
             raise ValueError(_("Unknown Network Filesystem format: %s" % val))
         self._format = val
-    format = property(get_format, set_format)
+    format = property(get_format, set_format,
+                      doc=_("Type of network filesystem."))
 
     def _get_default_target_path(self):
         path = (DEFAULT_DIR_TARGET_BASE + self.name)
@@ -552,6 +547,9 @@ class LogicalPool(StoragePool):
 
     # Register applicable property methods from parent class
     perms = property(StorageObject.get_perms, StorageObject.set_perms)
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Location of the existing LVM volume group."))
 
     def __init__(self, conn, name, target_path=None, uuid=None, perms=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_LOGICAL,
@@ -577,18 +575,23 @@ class DiskPool(StoragePool):
 
     # Register applicable property methods from parent class
     source_path = property(StoragePool.get_source_path,
-                           StoragePool.set_source_path)
+                           StoragePool.set_source_path,
+                           doc=_("Path to the existing disk device."))
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Root location for identifying new storage"
+                                 " volumes."))
 
-    formats = [ "auto", "bsd", "msdos", "dvh", "gpt", "mac", "pc98", "sun" ]
+    formats = [ "auto", "bsd", "dos", "dvh", "gpt", "mac", "pc98", "sun" ]
 
     def get_volume_class():
-        return DiskVolume
+        raise NotImplementedError(_("Disk volume creation is not implemented."))
     get_volume_class = staticmethod(get_volume_class)
 
-    def __init__(self, conn, name, source_path=None, host=None,
-                 target_path=None, format="auto", uuid=None):
+    def __init__(self, conn, name, source_path=None, target_path=None,
+                 format="auto", uuid=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_DISK,
-                             uuid=None, target_path=target_path, conn=conn)
+                             uuid=uuid, target_path=target_path, conn=conn)
         self.format = format
         if source_path:
             self.source_path = source_path
@@ -599,7 +602,8 @@ class DiskPool(StoragePool):
         if not val in self.formats:
             raise ValueError(_("Unknown Disk format: %s" % val))
         self._format = val
-    format = property(get_format, set_format)
+    format = property(get_format, set_format,
+                      doc=_("Format of the source device's partition table."))
 
     def _get_default_target_path(self):
         return DEFAULT_DEV_TARGET
@@ -624,19 +628,30 @@ class DiskPool(StoragePool):
         if self.format == "auto" and build:
             raise ValueError(_("Must explicitly specify disk format if "
                                "formatting disk device."))
-        StoragePool.install(self, meter=meter, create=create, build=build)
+        return StoragePool.install(self, meter=meter, create=create,
+                                   build=build)
 
 class iSCSIPool(StoragePool):
     """
     Create an iSCSI based storage pool
     """
 
-    host = property(StoragePool.get_host, StoragePool.set_host)
+    host = property(StoragePool.get_host, StoragePool.set_host,
+                    doc=_("Name of the host sharing the storage."))
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Root location for identifying new storage"
+                                 " volumes."))
+
+    def get_volume_class():
+        raise NotImplementedError(_("iSCSI volume creation is not "
+                                    "implemented."))
+    get_volume_class = staticmethod(get_volume_class)
 
     def __init__(self, conn, name, source_path=None, host=None,
                  target_path=None, uuid=None):
         StoragePool.__init__(self, name=name, type=StoragePool.TYPE_ISCSI,
-                             uuid=None, target_path=target_path, conn=conn)
+                             uuid=uuid, target_path=target_path, conn=conn)
 
         if source_path:
             self.source_path = source_path
@@ -649,7 +664,8 @@ class iSCSIPool(StoragePool):
         return self._source_path
     def set_source_path(self, val):
         self._source_path = val
-    source_path = property(get_source_path, set_source_path)
+    source_path = property(get_source_path, set_source_path,
+                           doc=_("Path on the host that is being shared."))
 
     def _get_default_target_path(self):
         return DEFAULT_ISCSI_TARGET
@@ -700,7 +716,7 @@ class StorageVolume(StorageObject):
         pool_object = StorageVolume.lookup_pool_by_name(pool_object=pool_object,
                                                         pool_name=pool_name,
                                                         conn=conn)
-        return StoragePool.get_volume_for_pool(util.get_xml_path(pool_object.XMLDesc(0), "/pool/@type"))
+        return StoragePool.get_volume_for_pool(_util.get_xml_path(pool_object.XMLDesc(0), "/pool/@type"))
     get_volume_for_pool = staticmethod(get_volume_for_pool)
 
     def find_free_name(name, pool_object=None, pool_name=None, conn=None,
@@ -727,14 +743,13 @@ class StorageVolume(StorageObject):
                                                         conn=conn)
         pool_object.refresh(0)
 
-        vol = None
         for i in range(0, 100000):
             tryname = name
             if i != 0:
                 tryname += ("-%d" % i)
             tryname += suffix
             try:
-                vol = pool_object.storageVolLookupByName(tryname)
+                pool_object.storageVolLookupByName(tryname)
             except libvirt.libvirtError:
                 return tryname
         raise ValueError(_("Default volume target path range exceeded."))
@@ -752,7 +767,7 @@ class StorageVolume(StorageObject):
         if pool_name is not None and pool_object is None:
             if conn is None:
                 raise ValueError(_("'conn' must be specified with 'pool_name'"))
-            if not util.is_storage_capable(conn):
+            if not _util.is_storage_capable(conn):
                 raise ValueError(_("Connection does not support storage "
                                    "management."))
             try:
@@ -768,10 +783,6 @@ class StorageVolume(StorageObject):
     lookup_pool_by_name = staticmethod(lookup_pool_by_name)
 
 
-    def get_type(self):
-        return None
-    type = property(get_type)
-
     # Properties used by all volumes
     def get_capacity(self):
         return self._capacity
@@ -846,7 +857,7 @@ class StorageVolume(StorageObject):
         col = None
         try:
             col = self.conn.storageVolLookupByPath(path)
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError:
             pass
         if col:
             return True
@@ -860,7 +871,7 @@ class StorageVolume(StorageObject):
         raise RuntimeError, "Must be implemented in subclass"
 
     def _get_storage_xml(self):
-        src_xml = format_xml = ""
+        src_xml = ""
         if self._get_source_xml() != "":
             src_xml = "  <source>\n" + \
                       "%s" % (self._get_source_xml()) + \
@@ -951,12 +962,19 @@ class FileVolume(StorageVolume):
     def _get_source_xml(self):
         return ""
 
-class DiskVolume(StorageVolume):
-    """
-    Build and install xml for use on disk device pools
-    """
-    def __init__(self, *args, **kwargs):
-        raise RuntimeError ("Not Implemented")
+#class DiskVolume(StorageVolume):
+#    """
+#    Build and install xml for use on disk device pools
+#    """
+#    def __init__(self, *args, **kwargs):
+#        raise NotImplementedError
+
+#class iSCSIVolume(StorageVolume):
+#    """
+#    Build and install xml for use on iSCSI device pools
+#    """
+#    def __init__(self, *args, **kwargs):
+#        raise NotImplementedError
 
 class LogicalVolume(StorageVolume):
     """
