@@ -20,16 +20,30 @@
 # MA 02110-1301 USA.
 
 import os, stat, statvfs
+import subprocess
 import libxml2
 import logging
 import libvirt
-import __builtin__
 
-import util
+import _util
 import Storage
 from VirtualDevice import VirtualDevice
 from virtinst import _virtinst as _
 
+def _vdisk_create(path, size, kind, sparse = True):
+    force_fixed = "raw"
+    path = os.path.expanduser(path)
+    if kind in force_fixed or not sparse:
+        _type = kind + ":fixed"
+    else:
+        _type = kind + ":sparse"
+    try:
+        rc = subprocess.call([ '/usr/sbin/vdiskadm', 'create', '-t', _type,
+            '-s', str(size), path ])
+        return rc == 0
+    except OSError:
+        return False
+
 class VirtualDisk(VirtualDevice):
     """
     Builds a libvirt domain disk xml description
@@ -69,7 +83,9 @@ class VirtualDisk(VirtualDevice):
     DRIVER_TAP_RAW = "aio"
     DRIVER_TAP_QCOW = "qcow"
     DRIVER_TAP_VMDK = "vmdk"
-    driver_types = [DRIVER_TAP_RAW, DRIVER_TAP_QCOW, DRIVER_TAP_VMDK]
+    DRIVER_TAP_VDISK = "vdisk"
+    driver_types = [DRIVER_TAP_RAW, DRIVER_TAP_QCOW,
+        DRIVER_TAP_VMDK, DRIVER_TAP_VDISK]
 
     DEVICE_DISK = "disk"
     DEVICE_CDROM = "cdrom"
@@ -109,11 +125,31 @@ class VirtualDisk(VirtualDevice):
         @type volObject: libvirt.virStorageVol
         @param volInstall: StorageVolume instance to build for new storage
         @type volInstall: L{StorageVolume}
+        @param volName: Existing StorageVolume lookup information,
+                        (parent pool name, volume name)
+        @type volName: C{tuple} of (C{str}, C{str})
         @param bus: Emulated bus type (ide, scsi, virtio, ...)
         @type bus: C{str}
         """
 
         VirtualDevice.__init__(self, conn=conn)
+
+        self._path = None
+        self._size = None
+        self._type = None
+        self._device = None
+        self._sparse = None
+        self._readOnly = None
+        self._vol_object = None
+        self._vol_install = None
+        self._bus = None
+
+        # XXX: No property methods for these
+        self.transient = transient
+        self._driverName = driverName
+        self._driverType = driverType
+        self.target = None
+
         self.set_read_only(readOnly, validate=False)
         self.set_sparse(sparse, validate=False)
         self.set_type(type, validate=False)
@@ -124,11 +160,6 @@ class VirtualDisk(VirtualDevice):
         self._set_vol_install(volInstall, validate=False)
         self._set_bus(bus, validate=False)
 
-        self.transient = transient
-        self._driverName = driverName
-        self._driverType = driverType
-        self.target = None
-
         if volName:
             self.__lookup_vol_name(volName)
 
@@ -182,11 +213,15 @@ class VirtualDisk(VirtualDevice):
 
     def get_driver_name(self):
         return self._driverName
-    driver_name = property(get_driver_name)
+    def set_driver_name(self, val):
+        self._driverName = val
+    driver_name = property(get_driver_name, set_driver_name)
 
     def get_driver_type(self):
         return self._driverType
-    driver_type = property(get_driver_type)
+    def set_driver_type(self, val):
+        self._driverType = val
+    driver_type = property(get_driver_type, set_driver_type)
 
     def get_sparse(self):
         return self._sparse
@@ -228,6 +263,9 @@ class VirtualDisk(VirtualDevice):
     bus = property(_get_bus, _set_bus)
 
     # Validation assistance methods
+
+    # Initializes attribute if it hasn't been done, then validates args.
+    # If validation fails, reset attribute to original value and raise error
     def __validate_wrapper(self, varname, newval, validate=True):
         try:
             orig = getattr(self, varname)
@@ -243,7 +281,7 @@ class VirtualDisk(VirtualDevice):
 
     def __set_dev_type(self):
         """
-        Detect disk 'type' from passed storage parameters
+        Detect disk 'type' () from passed storage parameters
         """
 
         dtype = None
@@ -267,9 +305,13 @@ class VirtualDisk(VirtualDevice):
                 dtype = self.TYPE_BLOCK
             else:
                 dtype = self.TYPE_FILE
+            if _util.is_vdisk(self.path):
+                self._driverName = self.DRIVER_TAP
+                self._driverType = self.DRIVER_TAP_VDISK
 
-        logging.debug("Detected storage as type '%s'" % dtype)
-        if self.type is not None and dtype != self.type:
+        if self.type is None:
+            logging.debug("Detected storage as type '%s'" % dtype)
+        elif dtype != self.type:
             raise ValueError(_("Passed type '%s' does not match detected "
                                "storage type '%s'" % (self.type, dtype)))
         self.set_type(dtype, validate=False)
@@ -284,7 +326,7 @@ class VirtualDisk(VirtualDevice):
                                "('poolname', 'volname')"))
         if not self.conn:
             raise ValueError(_("'volName' requires a passed connection."))
-        if not util.is_storage_capable(self.conn):
+        if not _util.is_storage_capable(self.conn):
             raise ValueError(_("Connection does not support storage lookup."))
         try:
             pool = self.conn.storagePoolLookupByName(name_tuple[0])
@@ -300,6 +342,97 @@ class VirtualDisk(VirtualDevice):
         """
         return (self.vol_object != None or self.vol_install != None)
 
+    def __check_if_path_managed(self):
+        """
+        Determine if we can use libvirt storage apis to create or lookup
+        'self.path'
+        """
+        vol = None
+        verr = None
+        pool = _util.lookup_pool_by_path(self.conn,
+                                         os.path.dirname(self.path))
+        if pool:
+            # Is pool running?
+            if pool.info()[0] != libvirt.VIR_STORAGE_POOL_RUNNING:
+                pool = None
+
+            try:
+                vol = self.conn.storageVolLookupByPath(self.path)
+            except Exception, e:
+                try:
+                    try:
+                        # Pool may need to be refreshed, but if it errors,
+                        # invalidate it
+                        pool.refresh(0)
+                    except:
+                        pool = None
+                        raise
+                    vol = self.conn.storageVolLookupByPath(self.path)
+                except Exception, e:
+                    verr = str(e)
+
+        if not vol:
+            # Path wasn't a volume. See if base of path is a managed
+            # pool, and if so, setup a StorageVolume object
+            if pool:
+                if self.size == None:
+                    raise ValueError(_("Size must be specified for non "
+                                       "existent volume path '%s'" % \
+                                        self.path))
+                logging.debug("Path '%s' is target for pool '%s'. "
+                              "Creating volume '%s'." % \
+                              (os.path.dirname(self.path), pool.name(),
+                               os.path.basename(self.path)))
+                volclass = Storage.StorageVolume.get_volume_for_pool(pool_object=pool)
+                cap = (self.size * 1024 * 1024 * 1024)
+                if self.sparse:
+                    alloc = 0
+                else:
+                    #alloc = cap
+                    # XXX: disable setting managed storage as nonsparse
+                    # XXX: since it hoses libvirtd (for now)
+                    alloc = 0
+                vol = volclass(name=os.path.basename(self.path),
+                               capacity=cap, allocation=alloc, pool=pool)
+                self._set_vol_install(vol, validate=False)
+            elif self._is_remote():
+                raise ValueError(_("'%s' is not managed on remote "
+                                   "host: %s" % (self.path, verr)))
+        else:
+            self._set_vol_object(vol, validate=False)
+
+
+    def __sync_params(self):
+        """
+        Sync some parameters between storage objects and the older
+        VirtualDisk fields
+        """
+
+        newpath = None
+        if self.vol_object:
+            newpath = self.vol_object.path()
+        elif self.vol_install:
+            newpath = _util.get_xml_path(self.vol_install.pool.XMLDesc(0),
+                                         "/pool/target/path") + \
+                      self.vol_install.name
+
+        if newpath and newpath != self.path:
+            logging.debug("Overwriting 'path' from passed volume object.")
+            self._set_path(newpath, validate=False)
+
+        if self.vol_install:
+            newsize = self.vol_install.capacity/1024.0/1024.0/1024.0
+            if self.size != newsize:
+                logging.debug("Overwriting 'size' with value from "
+                              "StorageVolume")
+                self._set_size(newsize, validate=False)
+
+        # Remove this piece when storage volume creation is async
+        if self.sparse and self.vol_install and \
+           self.vol_install.allocation != 0:
+            logging.debug("Setting vol_install allocation to 0 (sparse).")
+            self.vol_install.allocation = 0
+
     def __validate_params(self):
         """
         function to validate all the complex interaction between the various
@@ -310,108 +443,62 @@ class VirtualDisk(VirtualDevice):
         # if no obj: if remote, error
         storage_capable = False
         if self.conn:
-            storage_capable = util.is_storage_capable(self.conn)
+            storage_capable = _util.is_storage_capable(self.conn)
+
         if not storage_capable and self._is_remote():
             raise ValueError, _("Connection doesn't support remote storage.")
 
-
+        # If the user didn't pass storage parameters, try to determine them
+        # from the passed path
         if storage_capable and self.path is not None \
            and not self.__storage_specified():
-            vol = None
-            verr = None
-            try:
-                vol = self.conn.storageVolLookupByPath(self.path)
-            except Exception, e:
-                verr = str(e)
+            self.__check_if_path_managed()
 
-            if not vol:
-                # Path wasn't a volume. See if base of path is a managed
-                # pool, and if so, setup a StorageVolume object
-                pool = util.lookup_pool_by_path(self.conn,
-                                                os.path.dirname(self.path))
-                if pool:
-                    if self.size == None:
-                        raise ValueError(_("Size must be specified for non "
-                                           "existent path '%s'" % self.path))
-                    logging.debug("Path '%s' is target for pool '%s'. "
-                                  "Creating volume '%s'." % \
-                                  (os.path.dirname(self.path), pool.name(),
-                                   os.path.basename(self.path)))
-                    volclass = Storage.StorageVolume.get_volume_for_pool(pool_object=pool)
-                    cap = (self.size * 1024 * 1024 * 1024)
-                    if self.sparse:
-                        alloc = 0
-                    else:
-                        #alloc = cap
-                        # XXX: disable setting managed storage as nonsparse
-                        # XXX: since it hoses libvirtd (for now)
-                        alloc = 0
-                    vol = volclass(name=os.path.basename(self.path),
-                                   capacity=cap, allocation=alloc,
-                                   pool=pool)
-                    self._set_vol_install(vol, validate=False)
-                elif self._is_remote():
-                    raise ValueError(_("'%s' is not managed on remote "
-                                       "host: %s" % (self.path, verr)))
-                else:
-                    logging.debug("Didn't find path '%s' managed on "
-                                  "connection: %s" % (self.path, verr))
-            else:
-                self._set_vol_object(vol, validate=False)
+        # Sync parameters between VirtualDisk and potentially passed
+        # storage objects.
+        self.__sync_params()
 
-
-        if self._is_remote() and not self.__storage_specified():
-            raise ValueError, _("Must specify libvirt managed storage if on "
-                                "a remote connection")
-
-        # Only floppy or cdrom can be created w/o media
-        if self.path is None and not self.__storage_specified():
+        # One small caveat: if self.path isn't set at this point, we are
+        # basically done.
+        if self.path is None:
             if self.device != self.DEVICE_FLOPPY and \
                self.device != self.DEVICE_CDROM:
                 raise ValueError, _("Device type '%s' requires a path") % \
                                   self.device
-            # If no path, our work is done
             return True
 
-        if self.vol_object:
-            logging.debug("Overwriting 'path' from passed volume object.")
-            self._set_path(self.vol_object.path(), validate=False)
 
-        if self.vol_install:
-            logging.debug("Overwriting 'size' with value from StorageVolume")
-            self._set_size(self.vol_install.capacity*1024*1024*1024,
-                           validate=False)
+        # The main distinctions from this point forward:
+        # Are we doing storage API operations or local media checks?
+        managed_storage = self.__storage_specified() or self.path is None
+        # Do we need to create the storage?
+        create_media = not ((managed_storage and self.vol_object) or \
+                            (self.path and os.path.exists(self.path)))
 
-        if self.__storage_specified():
-            logging.debug("Using storage api objects for VirtualDisk")
-            using_path = False
-        else:
-            logging.debug("Using self.path for VirtualDisk.")
-            using_path = True
+        if self._is_remote() and not managed_storage:
+            raise ValueError, _("Must specify libvirt managed storage if on "
+                                "a remote connection")
 
-        if self.sparse and self.vol_install and \
-           self.vol_install.allocation != 0:
-            logging.debug("Setting vol_install allocation to 0 (sparse).")
-            self.vol_install.allocation = 0
+        # If not creating the storage, our job is easy
+        if not create_media:
+            # Make sure we have access to the local path
+            if not managed_storage:
+                if os.path.isdir(self.path) and not _util.is_vdisk(self.path):
+                    # vdisk _is_ a directory.
+                    raise ValueError(_("The path '%s' must be a file or a "
+                                       "device, not a directory") % self.path)
+                # XXX: Any selinux validation checks should go here
 
-        if ((using_path and os.path.exists(self.path))
-                        or self.vol_object):
-            logging.debug("VirtualDisk storage exists.")
-
-            if using_path and os.path.isdir(self.path):
-                raise ValueError, _("The path must be a file or a device,"
-                                    " not a directory")
             self.__set_dev_type()
             return True
 
-        logging.debug("VirtualDisk storage does not exist.")
+
         if self.device == self.DEVICE_FLOPPY or \
            self.device == self.DEVICE_CDROM:
             raise ValueError, _("Cannot create storage for %s device.") % \
                                 self.device
 
-        if using_path:
-            # Not true for api?
+        if not managed_storage:
             if self.type is self.TYPE_BLOCK:
                 raise ValueError, _("Local block device path must exist.")
             self.set_type(self.TYPE_FILE, validate=False)
@@ -426,6 +513,7 @@ class VirtualDisk(VirtualDevice):
         else:
             self.__set_dev_type()
 
+        # Applicable for managed or local storage
         ret = self.is_size_conflict()
         if ret[0]:
             raise ValueError, ret[1]
@@ -450,20 +538,30 @@ class VirtualDisk(VirtualDevice):
             self._set_vol_object(self.vol_install.install(meter=progresscb),
                                  validate=False)
             return
-        elif self.type == VirtualDisk.TYPE_FILE and self.path is not None \
-             and not os.path.exists(self.path):
+        elif (self.type == VirtualDisk.TYPE_FILE and self.path is not None
+             and not os.path.exists(self.path)):
             size_bytes = long(self.size * 1024L * 1024L * 1024L)
 
             if progresscb:
                 progresscb.start(filename=self.path,size=long(size_bytes), \
                                  text=_("Creating storage file..."))
+
+            if _util.is_vdisk(self.path):
+                progresscb.update(1024)
+                if (not _vdisk_create(self.path, size_bytes, "vmdk",
+                    self.sparse)):
+                    raise RuntimeError, _("Error creating vdisk %s" % self.path)
+                self._driverName = self.DRIVER_TAP
+                self._driverType = self.DRIVER_TAP_VDISK
+                progresscb.end(self.size)
+                return
+
             fd = None
             try:
                 try:
                     fd = os.open(self.path, os.O_WRONLY | os.O_CREAT)
                     if self.sparse:
-                        os.lseek(fd, size_bytes, 0)
-                        os.write(fd, '\x00')
+                        os.ftruncate(fd, size_bytes)
                         if progresscb:
                             progresscb.update(self.size)
                     else:
@@ -503,7 +601,7 @@ class VirtualDisk(VirtualDevice):
         elif self.path:
             path = self.path
         if path:
-            path = util.xml_escape(path)
+            path = _util.xml_escape(path)
 
         ret = "    <disk type='%(type)s' device='%(device)s'>\n" % { "type": self.type, "device": self.device }
         if not(self.driver_name is None):
@@ -581,14 +679,14 @@ class VirtualDisk(VirtualDevice):
         """
         vms = []
         # get working domain's name
-        ids = conn.listDomainsID();
-        for id in ids:
+        ids = conn.listDomainsID()
+        for i in ids:
             try:
-                vm = conn.lookupByID(id)
+                vm = conn.lookupByID(i)
                 vms.append(vm)
             except libvirt.libvirtError:
                 # guest probably in process of dieing
-                logging.warn("Failed to lookup domain id %d" % id)
+                logging.warn("Failed to lookup domain id %d" % i)
         # get defined domain
         names = conn.listDefinedDomains()
         for name in names:
