@@ -2,24 +2,24 @@
 # Only use lower case entries: all lookups are .lower()'d
 keytable = {
     "ar":"ar",
-    "dk":"da", "da":"da",
+    "da":"da", "dk":"da",
     "de":"de",
     "de-ch":"de-ch",
-    "gb":"en-gb", "en-gb":"en-gb", "uk":"en-gb",
-    "us":"en-us", "en-us":"en-us",
+    "en-gb":"en-gb", "gb":"en-gb", "uk":"en-gb",
+    "en-us":"en-us", "us":"en-us",
     "es":"es",
     "et":"et",
-    "se_fi":"fi", "fi":"fi",
+    "fi":"fi", "se_fi":"fi",
     "fo":"fo",
     "fr":"fr",
-    "be":"fr-be","fr-be":"fr-be",
+    "fr-be":"fr-be", "be":"fr-be",
     "fr-ca":"fr-ca",
-    "fr_ch":"fr-ch", "fr-ch":"fr-ch",
+    "fr-ch":"fr-ch", "fr_ch":"fr-ch",
     "hr":"hr",
     "hu":"hu",
     "is":"is",
     "it":"it",
-    "jp106":"ja", "ja":"ja",
+    "ja":"ja", "jp106":"ja",
     "lt":"lt",
     "lv":"lv",
     "mk":"mk",
@@ -28,11 +28,11 @@ keytable = {
     "no":"no",
     "pl":"pl",
     "pt":"pt",
-    "br":"pt-br", "pt-br":"pt-br", "br-abnt2":"pt-br",
+    "pt-br":"pt-br", "br":"pt-br", "br-abnt2":"pt-br",
     "ru":"ru",
     "sl":"sl",
     "sv":"sv",
     "th":"th",
     "tr":"tr",
-    }
+}
 
